provider libmemcached {
	probe memcached_delete_start();
	probe memcached_delete_end();
	probe memcached_increment_with_initial_start();
	probe memcached_increment_with_initial_end();
	probe memcached_decrement_with_initial_start();
	probe memcached_decrement_with_initial_end();
	probe memcached_increment_start();
	probe memcached_increment_end();
	probe memcached_decrement_start();
	probe memcached_decrement_end();
	probe memcached_flush_start();
	probe memcached_flush_end();
	probe memcached_set_start();
	probe memcached_set_end();
	probe memcached_add_start();
	probe memcached_add_end();
	probe memcached_replace_start();
	probe memcached_replace_end();
	probe memcached_get_start();
	probe memcached_get_end();
 	probe memcached_touch_start();
 	probe memcached_touch_end();
	probe memcached_mget_start();
	probe memcached_mget_end();
	probe memcached_connect_start();
	probe memcached_connect_end();
	probe memcached_server_add_start();
	probe memcached_server_add_end();
};
