#line 2 "libmemcached/csl/scanner.cc"
#line 22 "libmemcached/csl/scanner.l"

#include <libmemcached/csl/common.h>
#include <libmemcached/csl/context.h>
#include <libmemcached/csl/parser.h>
#include <libmemcached/csl/symbol.h>

#ifndef __INTEL_COMPILER
#pragma GCC diagnostic ignored "-Wold-style-cast"
#pragma GCC diagnostic ignored "-Wsign-compare"
#pragma GCC diagnostic ignored "-Wunused-parameter"
#endif

#define YY_NO_INPUT

#define YY_EXTRA_TYPE Context*




#line 23 "libmemcached/csl/scanner.cc"

#define  YY_INT_ALIGNED short int

/* A lexical scanner generated by flex */

/* %not-for-header */

/* %if-c-only */
/* %if-not-reentrant */
/* %endif */
/* %endif */
/* %ok-for-header */

#define FLEX_SCANNER
#define YY_FLEX_MAJOR_VERSION 2
#define YY_FLEX_MINOR_VERSION 5
#define YY_FLEX_SUBMINOR_VERSION 35
#if YY_FLEX_SUBMINOR_VERSION > 0
#define FLEX_BETA
#endif

/* %if-c++-only */
/* %endif */

/* %if-c-only */
    
/* %endif */

/* %if-c-only */

/* %endif */

/* First, we deal with  platform-specific or compiler-specific issues. */

/* begin standard C headers. */
/* %if-c-only */
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
/* %endif */

/* %if-tables-serialization */
/* %endif */
/* end standard C headers. */

/* %if-c-or-c++ */
/* flex integer type definitions */

#ifndef FLEXINT_H
#define FLEXINT_H

/* C99 systems have <inttypes.h>. Non-C99 systems may or may not. */

#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L

/* C99 says to define __STDC_LIMIT_MACROS before including stdint.h,
 * if you want the limit (max/min) macros for int types. 
 */
#ifndef __STDC_LIMIT_MACROS
#define __STDC_LIMIT_MACROS 1
#endif

#include <inttypes.h>
typedef int8_t flex_int8_t;
typedef uint8_t flex_uint8_t;
typedef int16_t flex_int16_t;
typedef uint16_t flex_uint16_t;
typedef int32_t flex_int32_t;
typedef uint32_t flex_uint32_t;
#else
typedef signed char flex_int8_t;
typedef short int flex_int16_t;
typedef int flex_int32_t;
typedef unsigned char flex_uint8_t; 
typedef unsigned short int flex_uint16_t;
typedef unsigned int flex_uint32_t;
#endif /* ! C99 */

/* Limits of integral types. */
#ifndef INT8_MIN
#define INT8_MIN               (-128)
#endif
#ifndef INT16_MIN
#define INT16_MIN              (-32767-1)
#endif
#ifndef INT32_MIN
#define INT32_MIN              (-2147483647-1)
#endif
#ifndef INT8_MAX
#define INT8_MAX               (127)
#endif
#ifndef INT16_MAX
#define INT16_MAX              (32767)
#endif
#ifndef INT32_MAX
#define INT32_MAX              (2147483647)
#endif
#ifndef UINT8_MAX
#define UINT8_MAX              (255U)
#endif
#ifndef UINT16_MAX
#define UINT16_MAX             (65535U)
#endif
#ifndef UINT32_MAX
#define UINT32_MAX             (4294967295U)
#endif

#endif /* ! FLEXINT_H */

/* %endif */

/* %if-c++-only */
/* %endif */

#ifdef __cplusplus

/* The "const" storage-class-modifier is valid. */
#define YY_USE_CONST

#else	/* ! __cplusplus */

/* C99 requires __STDC__ to be defined as 1. */
#if defined (__STDC__)

#define YY_USE_CONST

#endif	/* defined (__STDC__) */
#endif	/* ! __cplusplus */

#ifdef YY_USE_CONST
#define yyconst const
#else
#define yyconst
#endif

/* %not-for-header */

/* Returned upon end-of-file. */
#define YY_NULL 0
/* %ok-for-header */

/* %not-for-header */

/* Promotes a possibly negative, possibly signed char to an unsigned
 * integer for use as an array index.  If the signed char is negative,
 * we want to instead treat it as an 8-bit unsigned char, hence the
 * double cast.
 */
#define YY_SC_TO_UI(c) ((unsigned int) (unsigned char) c)
/* %ok-for-header */

/* %if-reentrant */

/* An opaque pointer. */
#ifndef YY_TYPEDEF_YY_SCANNER_T
#define YY_TYPEDEF_YY_SCANNER_T
typedef void* yyscan_t;
#endif

/* For convenience, these vars (plus the bison vars far below)
   are macros in the reentrant scanner. */
#define yyin yyg->yyin_r
#define yyout yyg->yyout_r
#define yyextra yyg->yyextra_r
#define yyleng yyg->yyleng_r
#define yytext yyg->yytext_r
#define yylineno (YY_CURRENT_BUFFER_LVALUE->yy_bs_lineno)
#define yycolumn (YY_CURRENT_BUFFER_LVALUE->yy_bs_column)
#define yy_flex_debug yyg->yy_flex_debug_r

/* %endif */

/* %if-not-reentrant */
/* %endif */

/* Enter a start condition.  This macro really ought to take a parameter,
 * but we do it the disgusting crufty way forced on us by the ()-less
 * definition of BEGIN.
 */
#define BEGIN yyg->yy_start = 1 + 2 *

/* Translate the current start state into a value that can be later handed
 * to BEGIN to return to the state.  The YYSTATE alias is for lex
 * compatibility.
 */
#define YY_START ((yyg->yy_start - 1) / 2)
#define YYSTATE YY_START

/* Action number for EOF rule of a given start state. */
#define YY_STATE_EOF(state) (YY_END_OF_BUFFER + state + 1)

/* Special action meaning "start processing a new file". */
#define YY_NEW_FILE config_restart(yyin ,yyscanner )

#define YY_END_OF_BUFFER_CHAR 0

/* Size of default input buffer. */
#ifndef YY_BUF_SIZE
#define YY_BUF_SIZE 16384
#endif

/* The state buf must be large enough to hold one state per character in the main buffer.
 */
#define YY_STATE_BUF_SIZE   ((YY_BUF_SIZE + 2) * sizeof(yy_state_type))

#ifndef YY_TYPEDEF_YY_BUFFER_STATE
#define YY_TYPEDEF_YY_BUFFER_STATE
typedef struct yy_buffer_state *YY_BUFFER_STATE;
#endif

/* %if-not-reentrant */
/* %endif */

/* %if-c-only */
/* %if-not-reentrant */
/* %endif */
/* %endif */

#define EOB_ACT_CONTINUE_SCAN 0
#define EOB_ACT_END_OF_FILE 1
#define EOB_ACT_LAST_MATCH 2

    #define YY_LESS_LINENO(n)
    
/* Return all but the first "n" matched characters back to the input stream. */
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		*yy_cp = yyg->yy_hold_char; \
		YY_RESTORE_YY_MORE_OFFSET \
		yyg->yy_c_buf_p = yy_cp = yy_bp + yyless_macro_arg - YY_MORE_ADJ; \
		YY_DO_BEFORE_ACTION; /* set up yytext again */ \
		} \
	while ( 0 )

#define unput(c) yyunput( c, yyg->yytext_ptr , yyscanner )

#ifndef YY_TYPEDEF_YY_SIZE_T
#define YY_TYPEDEF_YY_SIZE_T
typedef size_t yy_size_t;
#endif

#ifndef YY_STRUCT_YY_BUFFER_STATE
#define YY_STRUCT_YY_BUFFER_STATE
struct yy_buffer_state
	{
/* %if-c-only */
	FILE *yy_input_file;
/* %endif */

/* %if-c++-only */
/* %endif */

	char *yy_ch_buf;		/* input buffer */
	char *yy_buf_pos;		/* current position in input buffer */

	/* Size of input buffer in bytes, not including room for EOB
	 * characters.
	 */
	yy_size_t yy_buf_size;

	/* Number of characters read into yy_ch_buf, not including EOB
	 * characters.
	 */
	int yy_n_chars;

	/* Whether we "own" the buffer - i.e., we know we created it,
	 * and can realloc() it to grow it, and should free() it to
	 * delete it.
	 */
	int yy_is_our_buffer;

	/* Whether this is an "interactive" input source; if so, and
	 * if we're using stdio for input, then we want to use getc()
	 * instead of fread(), to make sure we stop fetching input after
	 * each newline.
	 */
	int yy_is_interactive;

	/* Whether we're considered to be at the beginning of a line.
	 * If so, '^' rules will be active on the next match, otherwise
	 * not.
	 */
	int yy_at_bol;

    int yy_bs_lineno; /**< The line count. */
    int yy_bs_column; /**< The column count. */
    
	/* Whether to try to fill the input buffer when we reach the
	 * end of it.
	 */
	int yy_fill_buffer;

	int yy_buffer_status;

#define YY_BUFFER_NEW 0
#define YY_BUFFER_NORMAL 1
	/* When an EOF's been seen but there's still some text to process
	 * then we mark the buffer as YY_EOF_PENDING, to indicate that we
	 * shouldn't try reading from the input source any more.  We might
	 * still have a bunch of tokens to match, though, because of
	 * possible backing-up.
	 *
	 * When we actually see the EOF, we change the status to "new"
	 * (via config_restart()), so that the user can continue scanning by
	 * just pointing yyin at a new input file.
	 */
#define YY_BUFFER_EOF_PENDING 2

	};
#endif /* !YY_STRUCT_YY_BUFFER_STATE */

/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */

/* %if-not-reentrant */
/* %endif */
/* %ok-for-header */

/* %endif */

/* We provide macros for accessing buffer states in case in the
 * future we want to put the buffer states in a more general
 * "scanner state".
 *
 * Returns the top of the stack, or NULL.
 */
#define YY_CURRENT_BUFFER ( yyg->yy_buffer_stack \
                          ? yyg->yy_buffer_stack[yyg->yy_buffer_stack_top] \
                          : NULL)

/* Same as previous macro, but useful when we know that the buffer stack is not
 * NULL or when we need an lvalue. For internal use only.
 */
#define YY_CURRENT_BUFFER_LVALUE yyg->yy_buffer_stack[yyg->yy_buffer_stack_top]

/* %if-c-only Standard (non-C++) definition */

/* %if-not-reentrant */
/* %not-for-header */

/* %ok-for-header */

/* %endif */

void config_restart (FILE *input_file ,yyscan_t yyscanner );
void config__switch_to_buffer (YY_BUFFER_STATE new_buffer ,yyscan_t yyscanner );
YY_BUFFER_STATE config__create_buffer (FILE *file,int size ,yyscan_t yyscanner );
void config__delete_buffer (YY_BUFFER_STATE b ,yyscan_t yyscanner );
void config__flush_buffer (YY_BUFFER_STATE b ,yyscan_t yyscanner );
void config_push_buffer_state (YY_BUFFER_STATE new_buffer ,yyscan_t yyscanner );
void config_pop_buffer_state (yyscan_t yyscanner );

static void config_ensure_buffer_stack (yyscan_t yyscanner );
static void config__load_buffer_state (yyscan_t yyscanner );
static void config__init_buffer (YY_BUFFER_STATE b,FILE *file ,yyscan_t yyscanner );

#define YY_FLUSH_BUFFER config__flush_buffer(YY_CURRENT_BUFFER ,yyscanner)

YY_BUFFER_STATE config__scan_buffer (char *base,yy_size_t size ,yyscan_t yyscanner );
YY_BUFFER_STATE config__scan_string (yyconst char *yy_str ,yyscan_t yyscanner );
YY_BUFFER_STATE config__scan_bytes (yyconst char *bytes,int len ,yyscan_t yyscanner );

/* %endif */

void *config_alloc (yy_size_t ,yyscan_t yyscanner );
void *config_realloc (void *,yy_size_t ,yyscan_t yyscanner );
void config_free (void * ,yyscan_t yyscanner );

#define yy_new_buffer config__create_buffer

#define yy_set_interactive(is_interactive) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){ \
        config_ensure_buffer_stack (yyscanner); \
		YY_CURRENT_BUFFER_LVALUE =    \
            config__create_buffer(yyin,YY_BUF_SIZE ,yyscanner); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_is_interactive = is_interactive; \
	}

#define yy_set_bol(at_bol) \
	{ \
	if ( ! YY_CURRENT_BUFFER ){\
        config_ensure_buffer_stack (yyscanner); \
		YY_CURRENT_BUFFER_LVALUE =    \
            config__create_buffer(yyin,YY_BUF_SIZE ,yyscanner); \
	} \
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = at_bol; \
	}

#define YY_AT_BOL() (YY_CURRENT_BUFFER_LVALUE->yy_at_bol)

/* %% [1.0] yytext/yyin/yyout/yy_state_type/yylineno etc. def's & init go here */
/* Begin user sect3 */

#define config_wrap(n) 1
#define YY_SKIP_YYWRAP

#define FLEX_DEBUG

typedef unsigned char YY_CHAR;

typedef int yy_state_type;

#define yytext_ptr yytext_r

/* %if-c-only Standard (non-C++) definition */

static yy_state_type yy_get_previous_state (yyscan_t yyscanner );
static yy_state_type yy_try_NUL_trans (yy_state_type current_state  ,yyscan_t yyscanner);
static int yy_get_next_buffer (yyscan_t yyscanner );
static void yy_fatal_error (yyconst char msg[] ,yyscan_t yyscanner );

/* %endif */

/* Done after the current pattern has been matched and before the
 * corresponding action - sets up yytext.
 */
#define YY_DO_BEFORE_ACTION \
	yyg->yytext_ptr = yy_bp; \
/* %% [2.0] code to fiddle yytext and yyleng for yymore() goes here \ */\
	yyleng = (size_t) (yy_cp - yy_bp); \
	yyg->yy_hold_char = *yy_cp; \
	*yy_cp = '\0'; \
/* %% [3.0] code to copy yytext_ptr to yytext[] goes here, if %array \ */\
	yyg->yy_c_buf_p = yy_cp;

/* %% [4.0] data tables for the DFA and the user's section 1 definitions go here */
#define YY_NUM_RULES 65
#define YY_END_OF_BUFFER 66
/* This struct is not used in this scanner,
   but its presence is necessary. */
struct yy_trans_info
	{
	flex_int32_t yy_verify;
	flex_int32_t yy_nxt;
	};
static yyconst flex_int16_t yy_accept[521] =
    {   0,
        0,    0,   66,   64,    5,    5,    1,   64,   64,   64,
        2,   64,   64,   64,   64,   64,   64,   64,   64,   64,
       64,   64,   64,   64,   64,   64,    0,   63,    0,   48,
        0,    0,    0,    2,    3,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    6,   48,   48,   48,   48,   48,
       48,   48,   48,   48,   48,   48,   48,   48,    4,   62,
       62,    2,    3,   62,   53,   62,   44,   62,   62,   62,
       62,   62,   62,   63,    0,   62,   52,   62,   62,   62,
       62,   62,   62,   48,   48,   48,   48,   48,   48,   48,

       48,   48,   48,   48,   48,   48,   48,   48,   48,   48,
       48,   48,   48,    4,    0,   62,    2,    3,   62,   62,
       62,   62,   62,   62,   62,   62,   62,   62,   62,   62,
       62,   46,   48,   48,   48,   48,   48,    0,   48,   48,
       48,   48,   48,   48,   48,   48,   48,   48,   48,   48,
       48,   48,   48,   48,   48,    4,   62,   62,    3,   62,
       42,   45,   47,   62,    0,   58,   62,   62,   62,   62,
       62,   41,   62,   48,   48,   48,   48,   48,   48,    0,
        0,    0,   48,   48,   48,   48,   48,   48,    0,   48,
       48,   48,    0,   48,   48,   48,    0,    0,   48,   48,

        4,    0,   62,    3,   62,    0,   62,   62,   62,   62,
       50,   59,   51,   62,   48,   48,   48,   48,   48,    0,
       15,    0,    0,    0,   48,   48,   48,    0,    0,   48,
        0,   48,   48,   48,    0,   48,    0,   48,    0,    0,
        0,    0,   48,    4,   61,   62,   62,   62,   62,   56,
       54,   40,   60,   43,   48,   48,   48,   48,   48,    0,
        0,    0,    0,   48,   48,   48,    0,    0,   48,    0,
       48,    0,   48,    0,   48,    0,   48,    0,    0,    0,
        0,   48,   61,   62,   57,   55,    0,    0,   48,   48,
       48,    0,    0,    0,    0,   48,   19,    0,    0,    0,

        0,   48,    0,    0,    0,    7,    0,    0,    8,    0,
       48,    0,    0,   34,    0,    0,   61,   62,    0,    0,
       48,    0,   48,    0,    0,    0,    0,   48,    0,    0,
        0,    0,   48,    0,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,   49,    0,    0,   48,    0,
       48,    0,    0,    0,    0,   48,    0,    0,   38,   37,
       48,    0,    0,    0,    0,    0,    0,    0,    0,    0,
        0,    0,   35,    0,    0,    0,    0,    0,   48,    0,
        0,    0,    0,   39,    0,    0,    0,    0,    0,    0,
        0,    0,    0,   29,    0,    0,    0,    0,   36,    0,

        0,    0,    0,   48,    0,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,    0,    0,   30,    0,    0,
       33,    0,    0,    0,    0,   48,    0,    0,    0,    0,
        0,    0,    0,   23,    0,    0,   26,    0,    0,    0,
       32,    0,    0,    0,    0,   13,    0,    0,    0,    0,
        0,   21,    0,    0,    0,    0,    0,   31,    0,    0,
        0,    0,    0,    0,    0,    0,    0,    0,    0,   25,
        0,    0,    9,   10,   11,    0,    0,    0,    0,    0,
        0,    0,    0,    0,    0,   12,    0,    0,   17,    0,
        0,    0,    0,    0,    0,    0,    0,   18,    0,    0,

        0,   27,   28,    0,    0,    0,    0,    0,   14,   16,
       20,    0,    0,    0,    0,    0,    0,   22,   24,    0
    } ;

static yyconst flex_int32_t yy_ec[256] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    2,    3,
        1,    1,    2,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    4,    1,    5,    6,    1,    1,    1,    1,    1,
        1,    1,    1,    7,    8,    9,   10,   11,   12,   13,
       14,   15,   16,   17,   11,   11,   11,   18,    1,    1,
       19,    1,   20,    1,   21,   22,   23,   24,   25,   26,
       27,   28,   29,   30,   31,   32,   33,   34,   35,   36,
       37,   38,   39,   40,   41,   42,   43,   44,   45,   46,
        1,   47,    1,    1,   48,    1,   49,   50,   51,   52,

       53,   54,   55,   56,   57,   58,   59,   60,   61,   62,
       63,   64,   65,   66,   67,   68,   69,   70,   71,   72,
       73,   74,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,

        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1
    } ;

static yyconst flex_int32_t yy_meta[75] =
    {   0,
        1,    1,    1,    1,    2,    1,    1,    2,    2,    1,
        2,    2,    2,    2,    2,    2,    2,    1,    1,    1,
        2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
        2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
        2,    2,    2,    2,    2,    2,    1,    2,    2,    2,
        2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
        2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
        2,    2,    2,    2
    } ;

static yyconst flex_int16_t yy_base[526] =
    {   0,
        0,  622,  626, 1711, 1711, 1711, 1711,   70,  617,  597,
       67,   74,    0,   57,   52,   59,   73,   57,   64,   74,
      610,   77,   79,   77,   65,  609,  101, 1711,  606,  138,
      202,  215,  217,  230,  237,  250,  252,  259,  265,  277,
      282,  287,  297,  271,  315,  377,  323,  342,  350,  359,
      364,  396,  401,  591, 1711,  449,  242,   73,   80,   89,
       76,  378,   78,  397,  398,   91,   76,   91,  441,  328,
      515,  528,  481,  499,  421,  541,  543,  548,  553,  559,
      565,  576,  590,  595,  649,    0,  597,  622,  614,  624,
      656,  632,  674,   85,  131,  124,  230,  249,  572,  260,

      272,  285,  403,  300,  319,  375,  336,  376,  650,  381,
      401,  415,  436,  717,  730,  743,  756,  768,  781,  787,
      789,  797,  803,  808,  825,  810,  838,  852,  854,  863,
      870,  702,  454,  455,  651,  442,  456,  656,  461,  534,
      538,  533,  537,  562,  571,  581,  628,  642,  570,  655,
      650,  656,  569,  687,  664,  893,  921,  872,  928,  907,
      879,  942,  944,  949,  951,  958,  967,  973,  975,  980,
      993,  995, 1000,  660,  683,  720,  750,  755,  126,  755,
      795,  786,  787,  805,  860,  567,  564,  856,  856,  855,
      941,  969,  971,  987,  547,  981,  996,  976,  546,  999,

     1048, 1062, 1075, 1711, 1088, 1090, 1024, 1097, 1106, 1112,
     1117, 1124, 1132, 1134, 1012, 1043, 1089, 1095, 1090, 1081,
     1711, 1087, 1085, 1107, 1099, 1116, 1111, 1110, 1120, 1124,
     1132, 1139,  527, 1128, 1138, 1128, 1141, 1132, 1146, 1139,
     1151, 1159, 1139, 1711, 1205, 1182, 1199, 1218, 1220, 1228,
     1233, 1235, 1240, 1246,  526,  517, 1165, 1192, 1217, 1217,
     1225,  514,  513, 1235, 1212,  472, 1229, 1240, 1230, 1229,
      471, 1223,  457, 1231,  258, 1230, 1230, 1246, 1248, 1237,
     1254,  456, 1297, 1310, 1312, 1317, 1243, 1257, 1258,  451,
     1263, 1287, 1289, 1293, 1288, 1310,    0, 1299, 1302, 1292,

     1304, 1293, 1315, 1315, 1313, 1711, 1318, 1308, 1711, 1309,
      406, 1313, 1325, 1711, 1311, 1321, 1376, 1378, 1316, 1348,
     1363, 1349, 1351, 1364,  403, 1355, 1373, 1370, 1371, 1374,
      373,  372, 1375, 1367, 1382, 1371, 1370, 1381, 1382, 1368,
     1386, 1418, 1378, 1391, 1388, 1406, 1394, 1405,  367, 1414,
     1415,  362, 1402, 1423, 1409,  346,  352, 1415, 1711, 1711,
      348, 1410, 1423, 1428, 1414, 1433, 1424, 1420, 1439, 1429,
     1438, 1442, 1711, 1419, 1425, 1425, 1442, 1436, 1439, 1448,
     1464, 1469, 1473, 1711, 1461, 1459, 1463, 1463, 1472, 1470,
     1466, 1465, 1484, 1711, 1470, 1482, 1480, 1469, 1711, 1480,

     1491, 1488, 1493, 1485, 1499, 1481, 1497, 1486, 1500, 1490,
     1513,  334, 1516, 1510,  333,  340,  336, 1711, 1512, 1530,
     1711, 1533, 1518, 1527, 1525,  322, 1528, 1537, 1523, 1531,
     1529,  321, 1531, 1711, 1544, 1530, 1711, 1532, 1533, 1548,
     1711, 1539, 1535, 1551, 1536, 1711, 1553, 1542, 1558, 1565,
     1562, 1711, 1565,  319,  289, 1578, 1581, 1711, 1579, 1573,
      285, 1573, 1576, 1583, 1589, 1580, 1590, 1591, 1582, 1711,
     1577, 1578, 1711, 1711, 1711,  282, 1590, 1606,  253, 1597,
     1606, 1607, 1606, 1607, 1608, 1711, 1613, 1598, 1711,  243,
     1616, 1621, 1612,  237,  221, 1630, 1632, 1711, 1627,  229,

     1625, 1711, 1711, 1643,  217,  208, 1631, 1646, 1711, 1711,
     1711, 1647, 1635, 1653, 1636, 1652,  202, 1711, 1711, 1711,
     1704,  154, 1706,  131, 1708
    } ;

static yyconst flex_int16_t yy_def[526] =
    {   0,
      520,    1,  520,  520,  520,  520,  520,  521,  520,  520,
      522,  520,  522,  522,  522,  522,  522,  522,  522,  522,
      522,  522,  522,  522,  522,  523,  521,  520,  521,  520,
      520,  524,  524,  524,  520,  524,  524,  524,  524,  524,
      524,  524,  524,  524,  524,  525,  524,  524,  524,  524,
      524,  524,  524,  523,  520,   30,   56,   56,   56,   56,
       56,   56,   56,   56,   56,   56,   56,   56,  520,  524,
      524,  524,  520,  524,  524,  524,  524,  524,  524,  524,
      524,  524,  524,  524,  525,   85,  524,  524,  524,  524,
      524,  524,  524,   56,   56,   56,   56,   56,   56,   56,

       56,   56,   56,   56,   56,   56,   56,   56,   56,   56,
       56,   56,   56,  520,  524,  524,  524,  520,  524,  524,
      524,  524,  524,  524,  524,  524,  524,  524,  524,  524,
      524,  524,   56,   56,   56,   56,   56,  520,   56,   56,
       56,   56,   56,   56,   56,   56,   56,   56,   56,   56,
       56,   56,   56,   56,   56,  520,  524,  524,  520,  524,
      524,  524,  524,  524,  524,  524,  524,  524,  524,  524,
      524,  524,  524,   56,   56,   56,   56,   56,   56,  520,
      520,  520,   56,   56,   56,   56,   56,   56,  520,   56,
       56,   56,  520,   56,   56,   56,  520,  520,   56,   56,

      520,  524,  524,  520,  524,  524,  524,  524,  524,  524,
      524,  524,  524,  524,   56,   56,   56,   56,   56,  520,
      520,  520,  520,  520,   56,   56,   56,  520,  520,   56,
      520,   56,   56,   56,  520,   56,  520,   56,  520,  520,
      520,  520,   56,  520,  524,  524,  524,  524,  524,  524,
      524,  524,  524,  524,   56,   56,   56,   56,   56,  520,
      520,  520,  520,   56,   56,   56,  520,  520,   56,  520,
       56,  520,   56,  520,   56,  520,   56,  520,  520,  520,
      520,   56,  524,  524,  524,  524,  520,  520,   56,   56,
       56,  520,  520,  520,  520,   56,   56,  520,  520,  520,

      520,   56,  520,  520,  520,  520,  520,  520,  520,  520,
       56,  520,  520,  520,  520,  520,  524,  524,  520,  520,
       56,  520,   56,  520,  520,  520,  520,   56,  520,  520,
      520,  520,   56,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  524,  520,  520,   56,  520,
       56,  520,  520,  520,  520,   56,  520,  520,  520,  520,
       56,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,   56,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,

      520,  520,  520,   56,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,   56,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,

      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,    0,
      520,  520,  520,  520,  520
    } ;

static yyconst flex_int16_t yy_nxt[1786] =
    {   0,
        4,    5,    6,    7,    8,    4,    7,    9,    4,   10,
       11,   11,   11,   11,   11,   11,   11,   12,    7,    4,
       13,   13,   14,   15,   16,   17,   13,   18,   19,   20,
       13,   21,   22,   13,   13,   13,   13,   23,   24,   25,
       13,   13,   13,   13,   13,   13,    4,    4,   13,   13,
       14,   15,   16,   17,   13,   18,   19,   20,   13,   21,
       22,   13,   13,   13,   13,   23,   24,   25,   13,   13,
       13,   13,   13,   13,   28,   33,   38,   34,   34,   34,
       34,   34,   34,   34,   35,   35,   35,   35,   35,   35,
       35,   36,   39,   41,   37,   43,   40,   44,   45,   50,

       47,   52,   53,   51,   38,   28,   42,   96,   97,   98,
       99,   48,  103,  111,  112,  113,   29,   49,  133,   36,
       39,   41,   37,   43,   40,   44,   45,   50,   47,   52,
       53,   51,   70,  220,   42,   96,   97,   98,   99,   48,
      103,  111,  112,  113,  221,   49,  133,   29,   56,   56,
       56,   56,   56,   56,   56,   32,  134,  135,   56,   57,
       58,   59,   56,   56,   56,   60,   61,   56,   56,   56,
       56,   62,   56,   63,   56,   64,   65,   66,   67,   68,
       56,   56,   56,   56,  134,  135,   56,   57,   58,   59,
       56,   56,   56,   60,   61,   56,   56,   56,   56,   62,

       56,   63,   56,   64,   65,   66,   67,   68,   56,   56,
       56,   56,   69,   69,   69,   69,   69,   69,   69,   32,
      519,   32,   32,   32,   32,   32,  511,   71,   71,   71,
       71,   71,   71,   71,   32,  510,  507,   32,   33,  503,
       72,   72,   72,   72,   72,   72,   72,   73,   73,   73,
       73,   73,   73,   73,   32,  502,   32,   32,   32,   32,
       32,  498,   32,   32,   32,  308,   32,   32,  136,   32,
       94,  489,   32,   32,   75,   32,  309,   32,   32,   32,
       76,   32,   95,   74,   32,   32,   32,  137,   77,   32,
       32,   32,  139,   82,   32,   32,  136,   32,   94,   32,

      486,   32,   75,  475,   32,   32,   32,  470,   76,  140,
       95,   74,   32,   79,   78,  137,   77,  141,   32,   32,
      139,   82,   32,   32,   32,   81,  469,   32,   80,   32,
       32,   32,   32,  144,   32,   32,   32,  140,   87,  452,
      446,   79,   78,  439,   32,  141,   32,  438,   83,   32,
       32,  437,  434,   81,   32,  387,   80,   32,   32,  385,
      145,  144,   32,   32,  384,   88,   32,   32,   32,  380,
       32,   32,   32,  148,  377,   32,   83,   27,   27,   27,
       27,   84,   27,   27,   85,   85,   27,   89,  145,   32,
      360,  359,   90,   88,   27,   27,   27,   32,  100,  149,

       32,  148,   91,   32,   32,   32,   32,  146,   32,   32,
      353,   32,  101,  341,  147,   89,  152,  104,  102,  105,
       90,  106,  107,   29,   85,   32,  100,  149,   32,   32,
       91,  108,  109,   92,  142,  146,  153,  143,  110,  154,
      101,   93,  147,   32,  152,  104,  102,  105,   32,  106,
      107,  114,  114,  114,  114,  114,  114,  114,  322,  108,
      109,   92,  142,  316,  153,  143,  110,  154,   32,   93,
       56,   56,   56,  155,  174,  306,   56,   56,  304,  298,
      175,  178,   56,  179,   56,  183,   56,   56,   56,   56,
       56,  118,  118,  118,  118,  118,  118,  118,   56,   56,

       56,  155,  174,   32,   56,   56,   32,   32,  175,  178,
       56,  179,   56,  183,   56,   56,   56,   56,   56,   32,
      295,  294,   32,  115,  288,  116,  116,  116,  116,  116,
      116,  116,   32,  287,  272,   32,   33,  119,  117,  117,
      117,  117,  117,  117,  117,   32,   32,   32,   32,   32,
       32,   32,   32,  242,  237,   32,   32,   32,  184,  185,
       32,   32,   32,   32,  186,  119,   32,   32,  187,   32,
      123,  229,   32,   32,  228,   32,  197,  193,  189,  138,
       32,  120,  121,   32,   32,  188,  184,  185,   32,  124,
       32,  122,  186,   55,   32,   32,  187,   32,   32,   32,

       32,   32,   32,   32,   32,   32,   32,  125,  520,  120,
      121,   55,   32,  188,   46,  190,   31,  124,   32,  122,
      126,   32,   32,   32,   30,  520,   32,   26,   32,   32,
       32,   32,   32,  520,  520,  125,   32,   32,  520,   32,
       32,  520,   32,  190,   32,  520,  128,  129,  126,   27,
       27,   27,   27,   84,   27,   27,   85,   85,   27,  520,
       32,   32,  127,   32,   32,  191,   27,   27,   27,   32,
      520,   32,  150,  131,  128,  129,  176,  180,   32,   32,
      130,   32,   32,  192,  177,  194,  181,  151,  182,  195,
      127,  196,  200,  191,  198,   29,   85,  215,  132,  520,

      150,  131,  520,   32,  176,  180,   32,  216,  130,   32,
       32,  192,  177,  194,  181,  151,  182,  195,  520,  196,
      200,   32,  520,  520,  199,  215,  132,  156,  156,  156,
      156,  156,  156,  156,   32,  216,  520,   32,   32,  520,
      157,  157,  157,  157,  157,  157,  157,   32,  217,   32,
       32,  115,  199,  158,  158,  158,  158,  158,  158,  158,
       32,  520,  520,   32,   32,  520,  117,  117,  117,  117,
      117,  117,  117,  520,  218,  520,  217,   32,  159,  159,
      159,  159,  159,  159,  159,   32,  520,  520,   32,   32,
       32,   32,  219,   32,   32,   32,   32,   32,  520,  222,

      520,   32,  218,   32,   32,   32,  520,   32,  520,  160,
       32,   32,   32,  161,   32,   32,   32,   32,   32,  223,
      219,  163,  520,  164,  224,  225,  162,  222,   32,   32,
      520,  520,   32,   32,   32,  166,   32,  160,  168,  520,
      226,  161,   32,  520,   32,   32,   32,  223,  520,  163,
      165,  164,  224,  225,  162,   32,   32,   32,   32,   32,
       32,   32,   32,  166,  520,  167,  168,   32,  226,  169,
       32,   32,   32,  520,   32,  520,   32,   32,   32,   32,
      115,  520,  520,   32,  227,   32,   32,   32,  171,  520,
      230,  520,  170,  167,  173,  231,  232,  169,  520,   32,

      520,   32,  172,  201,  201,  201,  201,  201,  201,  201,
       32,   32,  227,  520,   32,   32,  171,   32,  230,   32,
      170,  520,  173,  231,  232,   32,   32,  520,   32,  202,
      172,  203,  203,  203,  203,  203,  203,  203,  204,  204,
      204,  204,  204,  204,  204,  205,   32,  520,   32,   32,
       32,   32,   32,   32,   32,   32,   32,   32,   32,   32,
      520,  520,   32,  520,  207,   32,   32,  208,   32,  520,
      520,   32,  520,  205,   32,   32,  520,   32,  520,   32,
       32,   32,   32,   32,   32,  233,  520,   32,   32,   32,
      209,   32,  520,  234,  520,  211,  206,   32,   32,   32,

       32,   32,   32,   32,   32,   32,  210,   32,   32,  520,
      235,  236,  520,  233,   32,  238,  241,  212,  209,  520,
       32,  234,   32,  211,  243,  213,  239,   32,   32,  240,
      520,   32,   32,  520,  210,  520,  250,  214,  235,  236,
       32,  520,   32,  238,  241,  212,  520,   32,  520,  520,
      520,  520,  243,  213,  239,  520,  255,  240,  244,  244,
      244,  244,  244,  244,  244,  214,   32,  520,  520,   32,
       32,   32,  245,  245,  245,  245,  245,  245,  245,   32,
      256,  520,   32,  202,  255,  246,  246,  246,  246,  246,
      246,  246,   32,  520,   32,   32,   32,   32,   32,  520,

      520,   32,  520,  248,   32,   32,  249,  520,  256,   32,
       32,  251,  520,   32,   32,  257,   32,  258,  259,   32,
       32,   32,   32,  260,   32,   32,  261,  247,   32,  262,
      252,   32,   32,  263,  264,   32,   32,   32,   32,   32,
       32,   32,   32,  257,   32,  258,  259,  265,  266,  267,
      253,  260,  268,   32,  261,  247,  269,  262,  252,   32,
      270,  263,  264,  271,   32,  273,  274,  275,  276,  277,
      278,   32,  254,  279,  280,  265,  266,  267,  253,   32,
      268,   32,  281,  282,  269,  520,   32,  520,  270,   32,
      202,  271,  520,  273,  274,  275,  276,  277,  278,  520,

      254,  279,  280,   32,  520,  289,   32,   32,  520,   32,
      281,  282,   32,   32,  520,  283,  283,  283,  283,  283,
      283,  283,   32,  284,   32,   32,   32,   32,   32,   32,
      285,  290,   32,  289,  286,   32,   32,   32,  291,   32,
       32,   32,   32,   32,   32,  292,   32,   32,   32,  293,
       32,  284,   32,   32,   32,  296,  297,  299,  302,  290,
      300,  303,  305,  307,  310,   32,  291,   32,  301,  311,
      312,  313,  314,  292,  315,   32,  520,  293,  319,  520,
       32,  520,   32,  296,  297,  299,  302,   32,  300,  303,
      305,  307,  310,   32,  320,  321,  301,  311,  312,  313,

      314,   32,  315,  323,   32,   32,  319,  317,  317,  317,
      317,  317,  317,  317,   32,  520,   32,   32,   32,   32,
       32,   32,  320,  321,   32,   32,  324,  325,  326,  520,
      327,  323,  328,  329,  330,  331,  520,  332,  333,  334,
      335,  336,  337,  318,   32,  338,  339,  340,  342,  343,
      344,  345,  520,  347,  324,  325,  326,   32,  327,   32,
      328,  329,  330,  331,   32,  332,  333,  334,  335,  336,
      337,  318,  348,  338,  339,  340,  342,  343,  344,  345,
       32,  347,   32,   32,   32,   32,   32,  349,  350,  520,
      351,  352,  354,  355,  356,  520,  357,  520,  358,  361,

      348,  362,  363,  364,  365,  366,  367,  368,  369,  372,
       32,  373,  374,   32,   32,  349,  350,  346,  351,  352,
      354,  355,  356,   32,  357,   32,  358,  361,  375,  362,
      363,  364,  365,  366,  367,  368,  369,  372,  370,  373,
      374,  376,  378,  379,  381,  346,  371,  382,  383,  386,
      388,  389,  390,   32,  391,  392,  375,  393,  394,  395,
      396,  397,  398,  399,  400,  401,  370,  402,  403,  376,
      378,  379,  381,  404,  371,  382,  383,  386,  388,  389,
      390,  405,  391,  392,  406,  393,  394,  395,  396,  397,
      398,  399,  400,  401,  407,  402,  403,  408,  409,  410,

      411,  404,  412,  413,  414,  415,  416,  417,  418,  405,
      419,  420,  406,  421,  422,  423,  424,  425,  426,  427,
      428,  429,  407,  430,  431,  408,  409,  410,  411,  432,
      412,  413,  414,  415,  416,  417,  418,  433,  419,  420,
      435,  421,  422,  423,  424,  425,  426,  427,  428,  429,
      436,  430,  431,  440,  441,  442,  443,  432,  444,  445,
      447,  448,  449,  450,  451,  433,  453,  454,  435,  455,
      456,  457,  458,  459,  460,  461,  462,  463,  436,  464,
      465,  440,  441,  442,  443,  466,  444,  445,  447,  448,
      449,  450,  451,  467,  453,  454,  468,  455,  456,  457,

      458,  459,  460,  461,  462,  463,  471,  464,  465,  472,
      473,  474,  476,  466,  477,  478,  479,  480,  481,  482,
      483,  467,  484,  485,  468,  487,  488,  490,  491,  492,
      493,  494,  495,  496,  471,  497,  499,  472,  473,  474,
      476,  500,  477,  478,  479,  480,  481,  482,  483,  501,
      484,  485,  504,  487,  488,  490,  491,  492,  493,  494,
      495,  496,  505,  497,  499,  506,  508,  509,  512,  500,
      513,  514,  515,  516,  517,  518,  520,  501,  520,  520,
      504,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      505,  520,  520,  506,  508,  509,  512,  520,  513,  514,

      515,  516,  517,  518,   27,   27,   54,   54,   86,   86,
        3,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520
    } ;

static yyconst flex_int16_t yy_chk[1786] =
    {   0,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
        1,    1,    1,    1,    8,   11,   15,   11,   11,   11,
       11,   11,   11,   11,   12,   12,   12,   12,   12,   12,
       12,   14,   16,   17,   14,   18,   16,   19,   20,   23,

       22,   24,   25,   23,   15,   27,   17,   58,   59,   60,
       61,   22,   63,   66,   67,   68,    8,   22,   94,   14,
       16,   17,   14,   18,   16,   19,   20,   23,   22,   24,
       25,   23,  524,  179,   17,   58,   59,   60,   61,   22,
       63,   66,   67,   68,  179,   22,   94,   27,   30,   30,
       30,   30,   30,   30,   30,  522,   95,   96,   30,   30,
       30,   30,   30,   30,   30,   30,   30,   30,   30,   30,
       30,   30,   30,   30,   30,   30,   30,   30,   30,   30,
       30,   30,   30,   30,   95,   96,   30,   30,   30,   30,
       30,   30,   30,   30,   30,   30,   30,   30,   30,   30,

       30,   30,   30,   30,   30,   30,   30,   30,   30,   30,
       30,   30,   31,   31,   31,   31,   31,   31,   31,   32,
      517,   33,   32,   32,   33,   33,  506,   33,   33,   33,
       33,   33,   33,   33,   34,  505,  500,   34,   34,  495,
       34,   34,   34,   34,   34,   34,   34,   35,   35,   35,
       35,   35,   35,   35,   36,  494,   37,   36,   36,   37,
       37,  490,   32,   38,   33,  275,   38,   38,   97,   39,
       57,  479,   39,   39,   37,   44,  275,   34,   44,   44,
       38,   40,   57,   36,   40,   40,   41,   98,   39,   41,
       41,   42,  100,   44,   42,   42,   97,   36,   57,   37,

      476,   43,   37,  461,   43,   43,   38,  455,   38,  101,
       57,   36,   39,   41,   40,   98,   39,  102,   44,   45,
      100,   44,   45,   45,   40,   43,  454,   47,   42,   41,
       47,   47,   70,  104,   42,   70,   70,  101,   47,  432,
      426,   41,   40,  417,   43,  102,   48,  416,   45,   48,
       48,  415,  412,   43,   49,  361,   42,   49,   49,  357,
      105,  104,   45,   50,  356,   48,   50,   50,   51,  352,
       47,   51,   51,  107,  349,   70,   45,   46,   46,   46,
       46,   46,   46,   46,   46,   46,   46,   49,  105,   48,
      332,  331,   50,   48,   46,   46,   46,   49,   62,  108,

       52,  107,   51,   52,   52,   53,   50,  106,   53,   53,
      325,   51,   62,  311,  106,   49,  110,   64,   62,   64,
       50,   64,   65,   46,   46,   75,   62,  108,   75,   75,
       51,   65,   65,   52,  103,  106,  111,  103,   65,  112,
       62,   53,  106,   52,  110,   64,   62,   64,   53,   64,
       65,   69,   69,   69,   69,   69,   69,   69,  290,   65,
       65,   52,  103,  282,  111,  103,   65,  112,   75,   53,
       56,   56,   56,  113,  133,  273,   56,   56,  271,  266,
      134,  136,   56,  137,   56,  139,   56,   56,   56,   56,
       56,   73,   73,   73,   73,   73,   73,   73,   56,   56,

       56,  113,  133,   74,   56,   56,   74,   74,  134,  136,
       56,  137,   56,  139,   56,   56,   56,   56,   56,   71,
      263,  262,   71,   71,  256,   71,   71,   71,   71,   71,
       71,   71,   72,  255,  233,   72,   72,   74,   72,   72,
       72,   72,   72,   72,   72,   76,   74,   77,   76,   76,
       77,   77,   78,  199,  195,   78,   78,   79,  140,  141,
       79,   79,   71,   80,  142,   74,   80,   80,  143,   81,
       80,  187,   81,   81,  186,   72,  153,  149,  145,   99,
       82,   76,   78,   82,   82,  144,  140,  141,   76,   81,
       77,   79,  142,   54,   83,   78,  143,   83,   83,   84,

       79,   87,   84,   84,   87,   87,   80,   82,   29,   76,
       78,   26,   81,  144,   21,  146,   10,   81,   89,   79,
       83,   89,   89,   82,    9,    3,   88,    2,   90,   88,
       88,   90,   90,    0,    0,   82,   92,   83,    0,   92,
       92,    0,   84,  146,   87,    0,   89,   90,   83,   85,
       85,   85,   85,   85,   85,   85,   85,   85,   85,    0,
       91,   89,   88,   91,   91,  147,   85,   85,   85,   88,
        0,   90,  109,   92,   89,   90,  135,  138,   93,   92,
       91,   93,   93,  148,  135,  150,  138,  109,  138,  151,
       88,  152,  155,  147,  154,   85,   85,  174,   93,    0,

      109,   92,    0,   91,  135,  138,  132,  175,   91,  132,
      132,  148,  135,  150,  138,  109,  138,  151,    0,  152,
      155,   93,    0,    0,  154,  174,   93,  114,  114,  114,
      114,  114,  114,  114,  115,  175,    0,  115,  115,    0,
      115,  115,  115,  115,  115,  115,  115,  116,  176,  132,
      116,  116,  154,  116,  116,  116,  116,  116,  116,  116,
      117,    0,    0,  117,  117,    0,  117,  117,  117,  117,
      117,  117,  117,    0,  177,    0,  176,  115,  118,  118,
      118,  118,  118,  118,  118,  119,    0,    0,  119,  119,
      116,  120,  178,  121,  120,  120,  121,  121,    0,  180,

        0,  122,  177,  117,  122,  122,    0,  123,    0,  119,
      123,  123,  124,  120,  126,  124,  124,  126,  126,  181,
      178,  122,    0,  123,  182,  183,  121,  180,  119,  125,
        0,    0,  125,  125,  120,  124,  121,  119,  126,    0,
      184,  120,  127,    0,  122,  127,  127,  181,    0,  122,
      123,  123,  182,  183,  121,  124,  128,  126,  129,  128,
      128,  129,  129,  124,    0,  125,  126,  130,  184,  127,
      130,  130,  125,    0,  131,    0,  158,  131,  131,  158,
      158,    0,    0,  161,  185,  127,  161,  161,  129,    0,
      188,    0,  128,  125,  131,  189,  190,  127,    0,  128,

        0,  129,  130,  156,  156,  156,  156,  156,  156,  156,
      130,  160,  185,    0,  160,  160,  129,  131,  188,  158,
      128,    0,  131,  189,  190,  157,  161,    0,  157,  157,
      130,  157,  157,  157,  157,  157,  157,  157,  159,  159,
      159,  159,  159,  159,  159,  160,  162,    0,  163,  162,
      162,  163,  163,  164,  160,  165,  164,  164,  165,  165,
        0,    0,  166,    0,  165,  166,  166,  165,  157,    0,
        0,  167,    0,  160,  167,  167,    0,  168,    0,  169,
      168,  168,  169,  169,  170,  191,    0,  170,  170,  162,
      167,  163,    0,  192,    0,  169,  164,  171,  165,  172,

      171,  171,  172,  172,  173,  166,  168,  173,  173,    0,
      193,  194,    0,  191,  167,  196,  198,  170,  167,    0,
      168,  192,  169,  169,  200,  171,  197,  170,  207,  197,
        0,  207,  207,    0,  168,    0,  207,  173,  193,  194,
      171,    0,  172,  196,  198,  170,    0,  173,    0,    0,
        0,    0,  200,  171,  197,    0,  215,  197,  201,  201,
      201,  201,  201,  201,  201,  173,  202,    0,    0,  202,
      202,  207,  202,  202,  202,  202,  202,  202,  202,  203,
      216,    0,  203,  203,  215,  203,  203,  203,  203,  203,
      203,  203,  205,    0,  206,  205,  205,  206,  206,    0,

        0,  208,    0,  206,  208,  208,  206,    0,  216,  202,
      209,  208,    0,  209,  209,  217,  210,  218,  219,  210,
      210,  211,  203,  220,  211,  211,  222,  205,  212,  223,
      209,  212,  212,  224,  225,  205,  213,  206,  214,  213,
      213,  214,  214,  217,  208,  218,  219,  226,  227,  228,
      210,  220,  229,  209,  222,  205,  230,  223,  209,  210,
      231,  224,  225,  232,  211,  234,  235,  236,  237,  238,
      239,  212,  214,  240,  241,  226,  227,  228,  210,  213,
      229,  214,  242,  243,  230,    0,  246,    0,  231,  246,
      246,  232,    0,  234,  235,  236,  237,  238,  239,    0,

      214,  240,  241,  247,    0,  257,  247,  247,    0,  245,
      242,  243,  245,  245,    0,  245,  245,  245,  245,  245,
      245,  245,  248,  247,  249,  248,  248,  249,  249,  246,
      248,  258,  250,  257,  249,  250,  250,  251,  259,  252,
      251,  251,  252,  252,  253,  260,  247,  253,  253,  261,
      254,  247,  245,  254,  254,  264,  265,  267,  269,  258,
      268,  270,  272,  274,  276,  248,  259,  249,  268,  277,
      278,  279,  280,  260,  281,  250,    0,  261,  287,    0,
      251,    0,  252,  264,  265,  267,  269,  253,  268,  270,
      272,  274,  276,  254,  288,  289,  268,  277,  278,  279,

      280,  283,  281,  291,  283,  283,  287,  283,  283,  283,
      283,  283,  283,  283,  284,    0,  285,  284,  284,  285,
      285,  286,  288,  289,  286,  286,  292,  293,  294,    0,
      295,  291,  296,  298,  299,  300,    0,  301,  302,  303,
      304,  305,  307,  284,  283,  308,  308,  310,  312,  313,
      315,  316,    0,  319,  292,  293,  294,  284,  295,  285,
      296,  298,  299,  300,  286,  301,  302,  303,  304,  305,
      307,  284,  320,  308,  308,  310,  312,  313,  315,  316,
      317,  319,  318,  317,  317,  318,  318,  321,  322,    0,
      323,  324,  326,  327,  328,    0,  329,    0,  330,  333,

      320,  334,  335,  336,  337,  338,  339,  340,  341,  343,
      346,  344,  345,  346,  346,  321,  322,  318,  323,  324,
      326,  327,  328,  317,  329,  318,  330,  333,  347,  334,
      335,  336,  337,  338,  339,  340,  341,  343,  342,  344,
      345,  348,  350,  351,  353,  318,  342,  354,  355,  358,
      362,  363,  364,  346,  365,  366,  347,  367,  368,  369,
      370,  371,  372,  374,  375,  376,  342,  377,  378,  348,
      350,  351,  353,  379,  342,  354,  355,  358,  362,  363,
      364,  380,  365,  366,  381,  367,  368,  369,  370,  371,
      372,  374,  375,  376,  382,  377,  378,  383,  385,  386,

      387,  379,  388,  389,  390,  391,  392,  393,  395,  380,
      396,  397,  381,  398,  400,  401,  402,  403,  404,  405,
      406,  407,  382,  408,  409,  383,  385,  386,  387,  410,
      388,  389,  390,  391,  392,  393,  395,  411,  396,  397,
      413,  398,  400,  401,  402,  403,  404,  405,  406,  407,
      414,  408,  409,  419,  420,  422,  423,  410,  424,  425,
      427,  428,  429,  430,  431,  411,  433,  435,  413,  436,
      438,  439,  440,  442,  443,  444,  445,  447,  414,  448,
      449,  419,  420,  422,  423,  450,  424,  425,  427,  428,
      429,  430,  431,  451,  433,  435,  453,  436,  438,  439,

      440,  442,  443,  444,  445,  447,  456,  448,  449,  457,
      459,  460,  462,  450,  463,  464,  465,  466,  467,  468,
      469,  451,  471,  472,  453,  477,  478,  480,  481,  482,
      483,  484,  485,  487,  456,  488,  491,  457,  459,  460,
      462,  492,  463,  464,  465,  466,  467,  468,  469,  493,
      471,  472,  496,  477,  478,  480,  481,  482,  483,  484,
      485,  487,  497,  488,  491,  499,  501,  504,  507,  492,
      508,  512,  513,  514,  515,  516,    0,  493,    0,    0,
      496,    0,    0,    0,    0,    0,    0,    0,    0,    0,
      497,    0,    0,  499,  501,  504,  507,    0,  508,  512,

      513,  514,  515,  516,  521,  521,  523,  523,  525,  525,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520,  520,  520,  520,  520,  520,
      520,  520,  520,  520,  520
    } ;

static yyconst flex_int16_t yy_rule_linenum[65] =
    {   0,
       81,   83,   85,   87,   89,   92,   96,   98,  100,  101,
      102,  103,  104,  105,  106,  107,  108,  109,  110,  111,
      112,  113,  114,  115,  116,  117,  118,  119,  120,  121,
      122,  123,  124,  125,  126,  127,  129,  130,  132,  134,
      135,  136,  137,  138,  139,  141,  142,  145,  150,  151,
      152,  154,  155,  156,  157,  158,  159,  160,  161,  162,
      164,  173,  191,  198
    } ;

/* The intent behind this definition is that it'll catch
 * any uses of REJECT which flex missed.
 */
#define REJECT reject_used_but_not_detected
#define yymore() yymore_used_but_not_detected
#define YY_MORE_ADJ 0
#define YY_RESTORE_YY_MORE_OFFSET
#line 1 "libmemcached/csl/scanner.l"
/*  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
 *
 *  Configure Scripting Language
 *
 *  Copyright (C) 2011 DataDifferental, http://datadifferential.com
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Affero General Public License as
 *  published by the Free Software Foundation, either version 3 of the
 *  License, or (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Affero General Public License for more details.
 * 
 *  You should have received a copy of the GNU Affero General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#line 42 "libmemcached/csl/scanner.l"
#include <cstdlib>
#include <cstring>

#define PARAM config_get_extra(yyscanner)

#define get_lex_chars(buffer, result, max_size, context) \
{ \
  if (context->pos >= context->length) \
  { \
    result= YY_NULL; \
  } \
  else \
  { \
    result= (int)(context->length - context->pos); \
    (size_t)result > (size_t)max_size ? result= max_size : 0; \
    memcpy(buffer, context->buf + context->pos, result); \
    context->pos += result; \
  } \
}


#define YY_INPUT(buffer, result, max_size) get_lex_chars(buffer, result, max_size, PARAM)

#line 1158 "libmemcached/csl/scanner.cc"

#define INITIAL 0

#ifndef YY_NO_UNISTD_H
/* Special case for "unistd.h", since it is non-ANSI. We include it way
 * down here because we want the user's section 1 to have been scanned first.
 * The user has a chance to override it with an option.
 */
/* %if-c-only */
#include <unistd.h>
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

#ifndef YY_EXTRA_TYPE
#define YY_EXTRA_TYPE void *
#endif

/* %if-c-only Reentrant structure and macros (non-C++). */
/* %if-reentrant */

/* Holds the entire state of the reentrant scanner. */
struct yyguts_t
    {

    /* User-defined. Not touched by flex. */
    YY_EXTRA_TYPE yyextra_r;

    /* The rest are the same as the globals declared in the non-reentrant scanner. */
    FILE *yyin_r, *yyout_r;
    size_t yy_buffer_stack_top; /**< index of top of stack. */
    size_t yy_buffer_stack_max; /**< capacity of stack. */
    YY_BUFFER_STATE * yy_buffer_stack; /**< Stack as an array. */
    char yy_hold_char;
    int yy_n_chars;
    int yyleng_r;
    char *yy_c_buf_p;
    int yy_init;
    int yy_start;
    int yy_did_buffer_switch_on_eof;
    int yy_start_stack_ptr;
    int yy_start_stack_depth;
    int *yy_start_stack;
    yy_state_type yy_last_accepting_state;
    char* yy_last_accepting_cpos;

    int yylineno_r;
    int yy_flex_debug_r;

    char *yytext_r;
    int yy_more_flag;
    int yy_more_len;

    YYSTYPE * yylval_r;

    }; /* end struct yyguts_t */

/* %if-c-only */

static int yy_init_globals (yyscan_t yyscanner );

/* %endif */

/* %if-reentrant */

    /* This must go here because YYSTYPE and YYLTYPE are included
     * from bison output in section 1.*/
    #    define yylval yyg->yylval_r
    
int config_lex_init (yyscan_t* scanner);

int config_lex_init_extra (YY_EXTRA_TYPE user_defined,yyscan_t* scanner);

/* %endif */

/* %endif End reentrant structures and macros. */

/* Accessor methods to globals.
   These are made visible to non-reentrant scanners for convenience. */

int config_lex_destroy (yyscan_t yyscanner );

int config_get_debug (yyscan_t yyscanner );

void config_set_debug (int debug_flag ,yyscan_t yyscanner );

YY_EXTRA_TYPE config_get_extra (yyscan_t yyscanner );

void config_set_extra (YY_EXTRA_TYPE user_defined ,yyscan_t yyscanner );

FILE *config_get_in (yyscan_t yyscanner );

void config_set_in  (FILE * in_str ,yyscan_t yyscanner );

FILE *config_get_out (yyscan_t yyscanner );

void config_set_out  (FILE * out_str ,yyscan_t yyscanner );

int config_get_leng (yyscan_t yyscanner );

char *config_get_text (yyscan_t yyscanner );

int config_get_lineno (yyscan_t yyscanner );

void config_set_lineno (int line_number ,yyscan_t yyscanner );

int config_get_column  (yyscan_t yyscanner );

void config_set_column (int column_no ,yyscan_t yyscanner );

/* %if-bison-bridge */

YYSTYPE * config_get_lval (yyscan_t yyscanner );

void config_set_lval (YYSTYPE * yylval_param ,yyscan_t yyscanner );

/* %endif */

/* Macros after this point can all be overridden by user definitions in
 * section 1.
 */

#ifndef YY_SKIP_YYWRAP
#ifdef __cplusplus
extern "C" int config_wrap (yyscan_t yyscanner );
#else
extern int config_wrap (yyscan_t yyscanner );
#endif
#endif

/* %not-for-header */

/* %ok-for-header */

/* %endif */

#ifndef yytext_ptr
static void yy_flex_strncpy (char *,yyconst char *,int ,yyscan_t yyscanner);
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * ,yyscan_t yyscanner);
#endif

#ifndef YY_NO_INPUT
/* %if-c-only Standard (non-C++) definition */
/* %not-for-header */

#ifdef __cplusplus
static int yyinput (yyscan_t yyscanner );
#else
static int input (yyscan_t yyscanner );
#endif
/* %ok-for-header */

/* %endif */
#endif

/* %if-c-only */

/* %endif */

/* Amount of stuff to slurp up with each read. */
#ifndef YY_READ_BUF_SIZE
#define YY_READ_BUF_SIZE 8192
#endif

/* Copy whatever the last rule matched to the standard output. */
#ifndef ECHO
/* %if-c-only Standard (non-C++) definition */
/* This used to be an fputs(), but since the string might contain NUL's,
 * we now use fwrite().
 */
#define ECHO do { if (fwrite( yytext, yyleng, 1, yyout )) {} } while (0)
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif

/* Gets input and stuffs it into "buf".  number of characters read, or YY_NULL,
 * is returned in "result".
 */
#ifndef YY_INPUT
#define YY_INPUT(buf,result,max_size) \
/* %% [5.0] fread()/read() definition of YY_INPUT goes here unless we're doing C++ \ */\
	if ( YY_CURRENT_BUFFER_LVALUE->yy_is_interactive ) \
		{ \
		int c = '*'; \
		unsigned n; \
		for ( n = 0; n < max_size && \
			     (c = getc( yyin )) != EOF && c != '\n'; ++n ) \
			buf[n] = (char) c; \
		if ( c == '\n' ) \
			buf[n++] = (char) c; \
		if ( c == EOF && ferror( yyin ) ) \
			YY_FATAL_ERROR( "input in flex scanner failed" ); \
		result = n; \
		} \
	else \
		{ \
		errno=0; \
		while ( (result = fread(buf, 1, max_size, yyin))==0 && ferror(yyin)) \
			{ \
			if( errno != EINTR) \
				{ \
				YY_FATAL_ERROR( "input in flex scanner failed" ); \
				break; \
				} \
			errno=0; \
			clearerr(yyin); \
			} \
		}\
\
/* %if-c++-only C++ definition \ */\
/* %endif */

#endif

/* No semi-colon after return; correct usage is to write "yyterminate();" -
 * we don't want an extra ';' after the "return" because that will cause
 * some compilers to complain about unreachable statements.
 */
#ifndef yyterminate
#define yyterminate() return YY_NULL
#endif

/* Number of entries by which start-condition stack grows. */
#ifndef YY_START_STACK_INCR
#define YY_START_STACK_INCR 25
#endif

/* Report a fatal error. */
#ifndef YY_FATAL_ERROR
/* %if-c-only */
#define YY_FATAL_ERROR(msg) yy_fatal_error( msg , yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
#endif

/* %if-tables-serialization structures and prototypes */
/* %not-for-header */

/* %ok-for-header */

/* %not-for-header */

/* %tables-yydmap generated elements */
/* %endif */
/* end tables serialization structures and prototypes */

/* %ok-for-header */

/* Default declaration of generated scanner - a define so the user can
 * easily add parameters.
 */
#ifndef YY_DECL
#define YY_DECL_IS_OURS 1
/* %if-c-only Standard (non-C++) definition */

extern int config_lex \
               (YYSTYPE * yylval_param ,yyscan_t yyscanner);

#define YY_DECL int config_lex \
               (YYSTYPE * yylval_param , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only C++ definition */
/* %endif */
#endif /* !YY_DECL */

/* Code executed at the beginning of each rule, after yytext and yyleng
 * have been set up.
 */
#ifndef YY_USER_ACTION
#define YY_USER_ACTION
#endif

/* Code executed at the end of each rule. */
#ifndef YY_BREAK
#define YY_BREAK break;
#endif

/* %% [6.0] YY_RULE_SETUP definition goes here */
#define YY_RULE_SETUP \
	if ( yyleng > 0 ) \
		YY_CURRENT_BUFFER_LVALUE->yy_at_bol = \
				(yytext[yyleng - 1] == '\n'); \
	YY_USER_ACTION

/* %not-for-header */

/** The main scanner function which does all the work.
 */
YY_DECL
{
	register yy_state_type yy_current_state;
	register char *yy_cp, *yy_bp;
	register int yy_act;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

/* %% [7.0] user's declarations go here */
#line 78 "libmemcached/csl/scanner.l"



#line 1465 "libmemcached/csl/scanner.cc"

    yylval = yylval_param;

	if ( !yyg->yy_init )
		{
		yyg->yy_init = 1;

#ifdef YY_USER_INIT
		YY_USER_INIT;
#endif

		if ( ! yyg->yy_start )
			yyg->yy_start = 1;	/* first start state */

		if ( ! yyin )
/* %if-c-only */
			yyin = stdin;
/* %endif */
/* %if-c++-only */
/* %endif */

		if ( ! yyout )
/* %if-c-only */
			yyout = stdout;
/* %endif */
/* %if-c++-only */
/* %endif */

		if ( ! YY_CURRENT_BUFFER ) {
			config_ensure_buffer_stack (yyscanner);
			YY_CURRENT_BUFFER_LVALUE =
				config__create_buffer(yyin,YY_BUF_SIZE ,yyscanner);
		}

		config__load_buffer_state(yyscanner );
		}

	while ( 1 )		/* loops until end-of-file is reached */
		{
/* %% [8.0] yymore()-related code goes here */
		yy_cp = yyg->yy_c_buf_p;

		/* Support of yytext. */
		*yy_cp = yyg->yy_hold_char;

		/* yy_bp points to the position in yy_ch_buf of the start of
		 * the current run.
		 */
		yy_bp = yy_cp;

/* %% [9.0] code to set up and find next match goes here */
		yy_current_state = yyg->yy_start;
		yy_current_state += YY_AT_BOL();
yy_match:
		do
			{
			register YY_CHAR yy_c = yy_ec[YY_SC_TO_UI(*yy_cp)];
			if ( yy_accept[yy_current_state] )
				{
				yyg->yy_last_accepting_state = yy_current_state;
				yyg->yy_last_accepting_cpos = yy_cp;
				}
			while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
				{
				yy_current_state = (int) yy_def[yy_current_state];
				if ( yy_current_state >= 521 )
					yy_c = yy_meta[(unsigned int) yy_c];
				}
			yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
			++yy_cp;
			}
		while ( yy_base[yy_current_state] != 1711 );

yy_find_action:
/* %% [10.0] code to find the action number goes here */
		yy_act = yy_accept[yy_current_state];
		if ( yy_act == 0 )
			{ /* have to back up */
			yy_cp = yyg->yy_last_accepting_cpos;
			yy_current_state = yyg->yy_last_accepting_state;
			yy_act = yy_accept[yy_current_state];
			}

		YY_DO_BEFORE_ACTION;

/* %% [11.0] code for yylineno update goes here */

do_action:	/* This label is used only to access EOF actions. */

/* %% [12.0] debug code goes here */
		if ( yy_flex_debug )
			{
			if ( yy_act == 0 )
				fprintf( stderr, "--scanner backing up\n" );
			else if ( yy_act < 65 )
				fprintf( stderr, "--accepting rule at line %ld (\"%s\")\n",
				         (long)yy_rule_linenum[yy_act], yytext );
			else if ( yy_act == 65 )
				fprintf( stderr, "--accepting default rule (\"%s\")\n",
				         yytext );
			else if ( yy_act == 66 )
				fprintf( stderr, "--(end of buffer or a NUL)\n" );
			else
				fprintf( stderr, "--EOF (start condition %d)\n", YY_START );
			}

		switch ( yy_act )
	{ /* beginning of action switch */
/* %% [13.0] actions go here */
			case 0: /* must back up */
			/* undo the effects of YY_DO_BEFORE_ACTION */
			*yy_cp = yyg->yy_hold_char;
			yy_cp = yyg->yy_last_accepting_cpos;
			yy_current_state = yyg->yy_last_accepting_state;
			goto yy_find_action;

case 1:
YY_RULE_SETUP
#line 81 "libmemcached/csl/scanner.l"
{ return yytext[0];}
	YY_BREAK
case 2:
YY_RULE_SETUP
#line 83 "libmemcached/csl/scanner.l"
{ yylval->number= atoi(yytext); return (NUMBER); }
	YY_BREAK
case 3:
YY_RULE_SETUP
#line 85 "libmemcached/csl/scanner.l"
{ yylval->number= atoi(yytext +1); return PORT; }
	YY_BREAK
case 4:
YY_RULE_SETUP
#line 87 "libmemcached/csl/scanner.l"
{ yylval->number= atoi(yytext +2); return WEIGHT_START; }
	YY_BREAK
case 5:
/* rule 5 can match eol */
YY_RULE_SETUP
#line 89 "libmemcached/csl/scanner.l"
; /* skip whitespace */
	YY_BREAK
case 6:
*yy_cp = yyg->yy_hold_char; /* undo effects of setting up yytext */
yyg->yy_c_buf_p = yy_cp -= 1;
YY_DO_BEFORE_ACTION; /* set up yytext again */
YY_RULE_SETUP
#line 92 "libmemcached/csl/scanner.l"
{
      return COMMENT;
    }
	YY_BREAK
case 7:
YY_RULE_SETUP
#line 96 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; yyextra->set_server(); return yyextra->previous_token= SERVER; }
	YY_BREAK
case 8:
YY_RULE_SETUP
#line 98 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= SOCKET; }
	YY_BREAK
case 9:
YY_RULE_SETUP
#line 100 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= BINARY_PROTOCOL; }
	YY_BREAK
case 10:
YY_RULE_SETUP
#line 101 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= BUFFER_REQUESTS; }
	YY_BREAK
case 11:
YY_RULE_SETUP
#line 102 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= CONFIGURE_FILE; }
	YY_BREAK
case 12:
YY_RULE_SETUP
#line 103 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= CONNECT_TIMEOUT; }
	YY_BREAK
case 13:
YY_RULE_SETUP
#line 104 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= DISTRIBUTION; }
	YY_BREAK
case 14:
YY_RULE_SETUP
#line 105 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= HASH_WITH_NAMESPACE; }
	YY_BREAK
case 15:
YY_RULE_SETUP
#line 106 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= HASH; }
	YY_BREAK
case 16:
YY_RULE_SETUP
#line 107 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= IO_BYTES_WATERMARK; }
	YY_BREAK
case 17:
YY_RULE_SETUP
#line 108 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= IO_KEY_PREFETCH; }
	YY_BREAK
case 18:
YY_RULE_SETUP
#line 109 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= IO_MSG_WATERMARK; }
	YY_BREAK
case 19:
YY_RULE_SETUP
#line 110 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= NOREPLY; }
	YY_BREAK
case 20:
YY_RULE_SETUP
#line 111 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= NUMBER_OF_REPLICAS; }
	YY_BREAK
case 21:
YY_RULE_SETUP
#line 112 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= POLL_TIMEOUT; }
	YY_BREAK
case 22:
YY_RULE_SETUP
#line 113 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= RANDOMIZE_REPLICA_READ; }
	YY_BREAK
case 23:
YY_RULE_SETUP
#line 114 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= RCV_TIMEOUT; }
	YY_BREAK
case 24:
YY_RULE_SETUP
#line 115 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= REMOVE_FAILED_SERVERS; }
	YY_BREAK
case 25:
YY_RULE_SETUP
#line 116 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= RETRY_TIMEOUT; }
	YY_BREAK
case 26:
YY_RULE_SETUP
#line 117 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= SND_TIMEOUT; }
	YY_BREAK
case 27:
YY_RULE_SETUP
#line 118 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= SOCKET_RECV_SIZE; }
	YY_BREAK
case 28:
YY_RULE_SETUP
#line 119 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= SOCKET_SEND_SIZE; }
	YY_BREAK
case 29:
YY_RULE_SETUP
#line 120 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= SORT_HOSTS; }
	YY_BREAK
case 30:
YY_RULE_SETUP
#line 121 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= SUPPORT_CAS; }
	YY_BREAK
case 31:
YY_RULE_SETUP
#line 122 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= _TCP_KEEPALIVE; }
	YY_BREAK
case 32:
YY_RULE_SETUP
#line 123 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= _TCP_KEEPIDLE; }
	YY_BREAK
case 33:
YY_RULE_SETUP
#line 124 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= _TCP_NODELAY; }
	YY_BREAK
case 34:
YY_RULE_SETUP
#line 125 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= USE_UDP; }
	YY_BREAK
case 35:
YY_RULE_SETUP
#line 126 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= USER_DATA; }
	YY_BREAK
case 36:
YY_RULE_SETUP
#line 127 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= VERIFY_KEY; }
	YY_BREAK
case 37:
YY_RULE_SETUP
#line 129 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= POOL_MIN; }
	YY_BREAK
case 38:
YY_RULE_SETUP
#line 130 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= POOL_MAX; }
	YY_BREAK
case 39:
YY_RULE_SETUP
#line 132 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= NAMESPACE; }
	YY_BREAK
case 40:
YY_RULE_SETUP
#line 134 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= INCLUDE; }
	YY_BREAK
case 41:
YY_RULE_SETUP
#line 135 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= RESET; }
	YY_BREAK
case 42:
YY_RULE_SETUP
#line 136 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= PARSER_DEBUG; }
	YY_BREAK
case 43:
YY_RULE_SETUP
#line 137 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= SERVERS; }
	YY_BREAK
case 44:
YY_RULE_SETUP
#line 138 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= END; }
	YY_BREAK
case 45:
YY_RULE_SETUP
#line 139 "libmemcached/csl/scanner.l"
{ yyextra->begin= yytext; return yyextra->previous_token= ERROR; }
	YY_BREAK
case 46:
YY_RULE_SETUP
#line 141 "libmemcached/csl/scanner.l"
{ return yyextra->previous_token= TRUE; }
	YY_BREAK
case 47:
YY_RULE_SETUP
#line 142 "libmemcached/csl/scanner.l"
{ return yyextra->previous_token= FALSE; }
	YY_BREAK
case 48:
YY_RULE_SETUP
#line 145 "libmemcached/csl/scanner.l"
{
      yyextra->begin= yytext;
      return UNKNOWN_OPTION;
    }
	YY_BREAK
case 49:
YY_RULE_SETUP
#line 150 "libmemcached/csl/scanner.l"
{ return CONSISTENT; }
	YY_BREAK
case 50:
YY_RULE_SETUP
#line 151 "libmemcached/csl/scanner.l"
{ return MODULA; }
	YY_BREAK
case 51:
YY_RULE_SETUP
#line 152 "libmemcached/csl/scanner.l"
{ return RANDOM; }
	YY_BREAK
case 52:
YY_RULE_SETUP
#line 154 "libmemcached/csl/scanner.l"
{ return MD5; }
	YY_BREAK
case 53:
YY_RULE_SETUP
#line 155 "libmemcached/csl/scanner.l"
{ return CRC; }
	YY_BREAK
case 54:
YY_RULE_SETUP
#line 156 "libmemcached/csl/scanner.l"
{ return FNV1_64; }
	YY_BREAK
case 55:
YY_RULE_SETUP
#line 157 "libmemcached/csl/scanner.l"
{ return FNV1A_64; }
	YY_BREAK
case 56:
YY_RULE_SETUP
#line 158 "libmemcached/csl/scanner.l"
{ return FNV1_32; }
	YY_BREAK
case 57:
YY_RULE_SETUP
#line 159 "libmemcached/csl/scanner.l"
{ return FNV1A_32; }
	YY_BREAK
case 58:
YY_RULE_SETUP
#line 160 "libmemcached/csl/scanner.l"
{ return HSIEH; }
	YY_BREAK
case 59:
YY_RULE_SETUP
#line 161 "libmemcached/csl/scanner.l"
{ return MURMUR; }
	YY_BREAK
case 60:
YY_RULE_SETUP
#line 162 "libmemcached/csl/scanner.l"
{ return JENKINS; }
	YY_BREAK
case 61:
YY_RULE_SETUP
#line 164 "libmemcached/csl/scanner.l"
{
      yylval->server.port= MEMCACHED_DEFAULT_PORT;
      yylval->server.weight= 1;
      yylval->server.c_str= yyextra->set_hostname(yytext, yyleng);
      if (yylval->server.c_str)
        yylval->server.size= yyleng;
      return IPADDRESS;
    }
	YY_BREAK
case 62:
YY_RULE_SETUP
#line 173 "libmemcached/csl/scanner.l"
{
      if (yyextra->is_server())
      {
        yylval->server.port= MEMCACHED_DEFAULT_PORT;
        yylval->server.weight= 1;
        yylval->server.c_str= yyextra->set_hostname(yytext, yyleng);
        if (yylval->server.c_str)
          yylval->server.size= yyleng;

        return HOSTNAME;
      }

      yylval->string.c_str = yytext;
      yylval->string.size = yyleng;

      return STRING;
    }
	YY_BREAK
case 63:
/* rule 63 can match eol */
YY_RULE_SETUP
#line 191 "libmemcached/csl/scanner.l"
{
      config_get_text(yyscanner)[yyleng -1]= 0;
      yylval->string.c_str= yytext +1;
      yylval->string.size= yyleng -2;
      return QUOTED_STRING;
    }
	YY_BREAK
case 64:
YY_RULE_SETUP
#line 198 "libmemcached/csl/scanner.l"
{
      yyextra->begin= yytext;
      return UNKNOWN;
    }
	YY_BREAK
case 65:
YY_RULE_SETUP
#line 203 "libmemcached/csl/scanner.l"
ECHO;
	YY_BREAK
#line 1948 "libmemcached/csl/scanner.cc"
case YY_STATE_EOF(INITIAL):
	yyterminate();

	case YY_END_OF_BUFFER:
		{
		/* Amount of text matched not including the EOB char. */
		int yy_amount_of_matched_text = (int) (yy_cp - yyg->yytext_ptr) - 1;

		/* Undo the effects of YY_DO_BEFORE_ACTION. */
		*yy_cp = yyg->yy_hold_char;
		YY_RESTORE_YY_MORE_OFFSET

		if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_NEW )
			{
			/* We're scanning a new file or input source.  It's
			 * possible that this happened because the user
			 * just pointed yyin at a new source and called
			 * config_lex().  If so, then we have to assure
			 * consistency between YY_CURRENT_BUFFER and our
			 * globals.  Here is the right place to do so, because
			 * this is the first action (other than possibly a
			 * back-up) that will match for the new input source.
			 */
			yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
			YY_CURRENT_BUFFER_LVALUE->yy_input_file = yyin;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status = YY_BUFFER_NORMAL;
			}

		/* Note that here we test for yy_c_buf_p "<=" to the position
		 * of the first EOB in the buffer, since yy_c_buf_p will
		 * already have been incremented past the NUL character
		 * (since all states make transitions on EOB to the
		 * end-of-buffer state).  Contrast this with the test
		 * in input().
		 */
		if ( yyg->yy_c_buf_p <= &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
			{ /* This was really a NUL. */
			yy_state_type yy_next_state;

			yyg->yy_c_buf_p = yyg->yytext_ptr + yy_amount_of_matched_text;

			yy_current_state = yy_get_previous_state( yyscanner );

			/* Okay, we're now positioned to make the NUL
			 * transition.  We couldn't have
			 * yy_get_previous_state() go ahead and do it
			 * for us because it doesn't know how to deal
			 * with the possibility of jamming (and we don't
			 * want to build jamming into it because then it
			 * will run more slowly).
			 */

			yy_next_state = yy_try_NUL_trans( yy_current_state , yyscanner);

			yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;

			if ( yy_next_state )
				{
				/* Consume the NUL. */
				yy_cp = ++yyg->yy_c_buf_p;
				yy_current_state = yy_next_state;
				goto yy_match;
				}

			else
				{
/* %% [14.0] code to do back-up for compressed tables and set up yy_cp goes here */
				yy_cp = yyg->yy_c_buf_p;
				goto yy_find_action;
				}
			}

		else switch ( yy_get_next_buffer( yyscanner ) )
			{
			case EOB_ACT_END_OF_FILE:
				{
				yyg->yy_did_buffer_switch_on_eof = 0;

				if ( config_wrap(yyscanner ) )
					{
					/* Note: because we've taken care in
					 * yy_get_next_buffer() to have set up
					 * yytext, we can now set up
					 * yy_c_buf_p so that if some total
					 * hoser (like flex itself) wants to
					 * call the scanner after we return the
					 * YY_NULL, it'll still work - another
					 * YY_NULL will get returned.
					 */
					yyg->yy_c_buf_p = yyg->yytext_ptr + YY_MORE_ADJ;

					yy_act = YY_STATE_EOF(YY_START);
					goto do_action;
					}

				else
					{
					if ( ! yyg->yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
					}
				break;
				}

			case EOB_ACT_CONTINUE_SCAN:
				yyg->yy_c_buf_p =
					yyg->yytext_ptr + yy_amount_of_matched_text;

				yy_current_state = yy_get_previous_state( yyscanner );

				yy_cp = yyg->yy_c_buf_p;
				yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
				goto yy_match;

			case EOB_ACT_LAST_MATCH:
				yyg->yy_c_buf_p =
				&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars];

				yy_current_state = yy_get_previous_state( yyscanner );

				yy_cp = yyg->yy_c_buf_p;
				yy_bp = yyg->yytext_ptr + YY_MORE_ADJ;
				goto yy_find_action;
			}
		break;
		}

	default:
		YY_FATAL_ERROR(
			"fatal flex scanner internal error--no action found" );
	} /* end of action switch */
		} /* end of scanning one token */
} /* end of config_lex */
/* %ok-for-header */

/* %if-c++-only */
/* %not-for-header */

/* %ok-for-header */

/* %endif */

/* yy_get_next_buffer - try to read in a new buffer
 *
 * Returns a code representing an action:
 *	EOB_ACT_LAST_MATCH -
 *	EOB_ACT_CONTINUE_SCAN - continue scanning from current position
 *	EOB_ACT_END_OF_FILE - end of file
 */
/* %if-c-only */
static int yy_get_next_buffer (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	register char *dest = YY_CURRENT_BUFFER_LVALUE->yy_ch_buf;
	register char *source = yyg->yytext_ptr;
	register int number_to_move, i;
	int ret_val;

	if ( yyg->yy_c_buf_p > &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] )
		YY_FATAL_ERROR(
		"fatal flex scanner internal error--end of buffer missed" );

	if ( YY_CURRENT_BUFFER_LVALUE->yy_fill_buffer == 0 )
		{ /* Don't try to fill the buffer, so this is an EOF. */
		if ( yyg->yy_c_buf_p - yyg->yytext_ptr - YY_MORE_ADJ == 1 )
			{
			/* We matched a single character, the EOB, so
			 * treat this as a final EOF.
			 */
			return EOB_ACT_END_OF_FILE;
			}

		else
			{
			/* We matched some text prior to the EOB, first
			 * process it.
			 */
			return EOB_ACT_LAST_MATCH;
			}
		}

	/* Try to read more data. */

	/* First move last chars to start of buffer. */
	number_to_move = (int) (yyg->yy_c_buf_p - yyg->yytext_ptr) - 1;

	for ( i = 0; i < number_to_move; ++i )
		*(dest++) = *(source++);

	if ( YY_CURRENT_BUFFER_LVALUE->yy_buffer_status == YY_BUFFER_EOF_PENDING )
		/* don't do the read, it's not guaranteed to return an EOF,
		 * just force an EOF
		 */
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars = 0;

	else
		{
			int num_to_read =
			YY_CURRENT_BUFFER_LVALUE->yy_buf_size - number_to_move - 1;

		while ( num_to_read <= 0 )
			{ /* Not enough room in the buffer - grow it. */

			/* just a shorter name for the current buffer */
			YY_BUFFER_STATE b = YY_CURRENT_BUFFER;

			int yy_c_buf_p_offset =
				(int) (yyg->yy_c_buf_p - b->yy_ch_buf);

			if ( b->yy_is_our_buffer )
				{
				int new_size = b->yy_buf_size * 2;

				if ( new_size <= 0 )
					b->yy_buf_size += b->yy_buf_size / 8;
				else
					b->yy_buf_size *= 2;

				b->yy_ch_buf = (char *)
					/* Include room in for 2 EOB chars. */
					config_realloc((void *) b->yy_ch_buf,b->yy_buf_size + 2 ,yyscanner );
				}
			else
				/* Can't grow it, we don't own it. */
				b->yy_ch_buf = 0;

			if ( ! b->yy_ch_buf )
				YY_FATAL_ERROR(
				"fatal error - scanner input buffer overflow" );

			yyg->yy_c_buf_p = &b->yy_ch_buf[yy_c_buf_p_offset];

			num_to_read = YY_CURRENT_BUFFER_LVALUE->yy_buf_size -
						number_to_move - 1;

			}

		if ( num_to_read > YY_READ_BUF_SIZE )
			num_to_read = YY_READ_BUF_SIZE;

		/* Read in more data. */
		YY_INPUT( (&YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[number_to_move]),
			yyg->yy_n_chars, (size_t) num_to_read );

		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	if ( yyg->yy_n_chars == 0 )
		{
		if ( number_to_move == YY_MORE_ADJ )
			{
			ret_val = EOB_ACT_END_OF_FILE;
			config_restart(yyin  ,yyscanner);
			}

		else
			{
			ret_val = EOB_ACT_LAST_MATCH;
			YY_CURRENT_BUFFER_LVALUE->yy_buffer_status =
				YY_BUFFER_EOF_PENDING;
			}
		}

	else
		ret_val = EOB_ACT_CONTINUE_SCAN;

	if ((yy_size_t) (yyg->yy_n_chars + number_to_move) > YY_CURRENT_BUFFER_LVALUE->yy_buf_size) {
		/* Extend the array by 50%, plus the number we really need. */
		yy_size_t new_size = yyg->yy_n_chars + number_to_move + (yyg->yy_n_chars >> 1);
		YY_CURRENT_BUFFER_LVALUE->yy_ch_buf = (char *) config_realloc((void *) YY_CURRENT_BUFFER_LVALUE->yy_ch_buf,new_size ,yyscanner );
		if ( ! YY_CURRENT_BUFFER_LVALUE->yy_ch_buf )
			YY_FATAL_ERROR( "out of dynamic memory in yy_get_next_buffer()" );
	}

	yyg->yy_n_chars += number_to_move;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] = YY_END_OF_BUFFER_CHAR;
	YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars + 1] = YY_END_OF_BUFFER_CHAR;

	yyg->yytext_ptr = &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[0];

	return ret_val;
}

/* yy_get_previous_state - get the state just before the EOB char was reached */

/* %if-c-only */
/* %not-for-header */

    static yy_state_type yy_get_previous_state (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	register yy_state_type yy_current_state;
	register char *yy_cp;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

/* %% [15.0] code to get the start state into yy_current_state goes here */
	yy_current_state = yyg->yy_start;
	yy_current_state += YY_AT_BOL();

	for ( yy_cp = yyg->yytext_ptr + YY_MORE_ADJ; yy_cp < yyg->yy_c_buf_p; ++yy_cp )
		{
/* %% [16.0] code to find the next state goes here */
		register YY_CHAR yy_c = (*yy_cp ? yy_ec[YY_SC_TO_UI(*yy_cp)] : 1);
		if ( yy_accept[yy_current_state] )
			{
			yyg->yy_last_accepting_state = yy_current_state;
			yyg->yy_last_accepting_cpos = yy_cp;
			}
		while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
			{
			yy_current_state = (int) yy_def[yy_current_state];
			if ( yy_current_state >= 521 )
				yy_c = yy_meta[(unsigned int) yy_c];
			}
		yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
		}

	return yy_current_state;
}

/* yy_try_NUL_trans - try to make a transition on the NUL character
 *
 * synopsis
 *	next_state = yy_try_NUL_trans( current_state );
 */
/* %if-c-only */
    static yy_state_type yy_try_NUL_trans  (yy_state_type yy_current_state , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	register int yy_is_jam;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner; /* This var may be unused depending upon options. */
/* %% [17.0] code to find the next state, and perhaps do backing up, goes here */
	register char *yy_cp = yyg->yy_c_buf_p;

	register YY_CHAR yy_c = 1;
	if ( yy_accept[yy_current_state] )
		{
		yyg->yy_last_accepting_state = yy_current_state;
		yyg->yy_last_accepting_cpos = yy_cp;
		}
	while ( yy_chk[yy_base[yy_current_state] + yy_c] != yy_current_state )
		{
		yy_current_state = (int) yy_def[yy_current_state];
		if ( yy_current_state >= 521 )
			yy_c = yy_meta[(unsigned int) yy_c];
		}
	yy_current_state = yy_nxt[yy_base[yy_current_state] + (unsigned int) yy_c];
	yy_is_jam = (yy_current_state == 520);

	return yy_is_jam ? 0 : yy_current_state;
}

/* %if-c-only */

/* %endif */

/* %if-c-only */
#ifndef YY_NO_INPUT
#ifdef __cplusplus
    static int yyinput (yyscan_t yyscanner)
#else
    static int input  (yyscan_t yyscanner)
#endif

/* %endif */
/* %if-c++-only */
/* %endif */
{
	int c;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	*yyg->yy_c_buf_p = yyg->yy_hold_char;

	if ( *yyg->yy_c_buf_p == YY_END_OF_BUFFER_CHAR )
		{
		/* yy_c_buf_p now points to the character we want to return.
		 * If this occurs *before* the EOB characters, then it's a
		 * valid NUL; if not, then we've hit the end of the buffer.
		 */
		if ( yyg->yy_c_buf_p < &YY_CURRENT_BUFFER_LVALUE->yy_ch_buf[yyg->yy_n_chars] )
			/* This was really a NUL. */
			*yyg->yy_c_buf_p = '\0';

		else
			{ /* need more input */
			int offset = yyg->yy_c_buf_p - yyg->yytext_ptr;
			++yyg->yy_c_buf_p;

			switch ( yy_get_next_buffer( yyscanner ) )
				{
				case EOB_ACT_LAST_MATCH:
					/* This happens because yy_g_n_b()
					 * sees that we've accumulated a
					 * token and flags that we need to
					 * try matching the token before
					 * proceeding.  But for input(),
					 * there's no matching to consider.
					 * So convert the EOB_ACT_LAST_MATCH
					 * to EOB_ACT_END_OF_FILE.
					 */

					/* Reset buffer status. */
					config_restart(yyin ,yyscanner);

					/*FALLTHROUGH*/

				case EOB_ACT_END_OF_FILE:
					{
					if ( config_wrap(yyscanner ) )
						return EOF;

					if ( ! yyg->yy_did_buffer_switch_on_eof )
						YY_NEW_FILE;
#ifdef __cplusplus
					return yyinput(yyscanner);
#else
					return input(yyscanner);
#endif
					}

				case EOB_ACT_CONTINUE_SCAN:
					yyg->yy_c_buf_p = yyg->yytext_ptr + offset;
					break;
				}
			}
		}

	c = *(unsigned char *) yyg->yy_c_buf_p;	/* cast for 8-bit char's */
	*yyg->yy_c_buf_p = '\0';	/* preserve yytext */
	yyg->yy_hold_char = *++yyg->yy_c_buf_p;

/* %% [19.0] update BOL and yylineno */
	YY_CURRENT_BUFFER_LVALUE->yy_at_bol = (c == '\n');

	return c;
}
/* %if-c-only */
#endif	/* ifndef YY_NO_INPUT */
/* %endif */

/** Immediately switch to a different input stream.
 * @param input_file A readable stream.
 * @param yyscanner The scanner object.
 * @note This function does not reset the start condition to @c INITIAL .
 */
/* %if-c-only */
    void config_restart  (FILE * input_file , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if ( ! YY_CURRENT_BUFFER ){
        config_ensure_buffer_stack (yyscanner);
		YY_CURRENT_BUFFER_LVALUE =
            config__create_buffer(yyin,YY_BUF_SIZE ,yyscanner);
	}

	config__init_buffer(YY_CURRENT_BUFFER,input_file ,yyscanner);
	config__load_buffer_state(yyscanner );
}

/** Switch to a different input buffer.
 * @param new_buffer The new input buffer.
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void config__switch_to_buffer  (YY_BUFFER_STATE  new_buffer , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	/* TODO. We should be able to replace this entire function body
	 * with
	 *		config_pop_buffer_state();
	 *		config_push_buffer_state(new_buffer);
     */
	config_ensure_buffer_stack (yyscanner);
	if ( YY_CURRENT_BUFFER == new_buffer )
		return;

	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*yyg->yy_c_buf_p = yyg->yy_hold_char;
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	YY_CURRENT_BUFFER_LVALUE = new_buffer;
	config__load_buffer_state(yyscanner );

	/* We don't actually know whether we did this switch during
	 * EOF (config_wrap()) processing, but the only time this flag
	 * is looked at is after config_wrap() is called, so it's safe
	 * to go ahead and always set it.
	 */
	yyg->yy_did_buffer_switch_on_eof = 1;
}

/* %if-c-only */
static void config__load_buffer_state  (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	yyg->yy_n_chars = YY_CURRENT_BUFFER_LVALUE->yy_n_chars;
	yyg->yytext_ptr = yyg->yy_c_buf_p = YY_CURRENT_BUFFER_LVALUE->yy_buf_pos;
	yyin = YY_CURRENT_BUFFER_LVALUE->yy_input_file;
	yyg->yy_hold_char = *yyg->yy_c_buf_p;
}

/** Allocate and initialize an input buffer state.
 * @param file A readable stream.
 * @param size The character buffer size in bytes. When in doubt, use @c YY_BUF_SIZE.
 * @param yyscanner The scanner object.
 * @return the allocated buffer state.
 */
/* %if-c-only */
    YY_BUFFER_STATE config__create_buffer  (FILE * file, int  size , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	YY_BUFFER_STATE b;
    
	b = (YY_BUFFER_STATE) config_alloc(sizeof( struct yy_buffer_state ) ,yyscanner );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in config__create_buffer()" );

	b->yy_buf_size = size;

	/* yy_ch_buf has to be 2 characters longer than the size given because
	 * we need to put in 2 end-of-buffer characters.
	 */
	b->yy_ch_buf = (char *) config_alloc(b->yy_buf_size + 2 ,yyscanner );
	if ( ! b->yy_ch_buf )
		YY_FATAL_ERROR( "out of dynamic memory in config__create_buffer()" );

	b->yy_is_our_buffer = 1;

	config__init_buffer(b,file ,yyscanner);

	return b;
}

/** Destroy the buffer.
 * @param b a buffer created with config__create_buffer()
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void config__delete_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if ( ! b )
		return;

	if ( b == YY_CURRENT_BUFFER ) /* Not sure if we should pop here. */
		YY_CURRENT_BUFFER_LVALUE = (YY_BUFFER_STATE) 0;

	if ( b->yy_is_our_buffer )
		config_free((void *) b->yy_ch_buf ,yyscanner );

	config_free((void *) b ,yyscanner );
}

/* %if-c-only */

#ifndef __cplusplus
extern int isatty (int );
#endif /* __cplusplus */
    
/* %endif */

/* %if-c++-only */
/* %endif */

/* Initializes or reinitializes a buffer.
 * This function is sometimes called more than once on the same buffer,
 * such as during a config_restart() or at EOF.
 */
/* %if-c-only */
    static void config__init_buffer  (YY_BUFFER_STATE  b, FILE * file , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */

{
	int oerrno = errno;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	config__flush_buffer(b ,yyscanner);

	b->yy_input_file = file;
	b->yy_fill_buffer = 1;

    /* If b is the current buffer, then config__init_buffer was _probably_
     * called from config_restart() or through yy_get_next_buffer.
     * In that case, we don't want to reset the lineno or column.
     */
    if (b != YY_CURRENT_BUFFER){
        b->yy_bs_lineno = 1;
        b->yy_bs_column = 0;
    }

/* %if-c-only */

        b->yy_is_interactive = file ? (isatty( fileno(file) ) > 0) : 0;
    
/* %endif */
/* %if-c++-only */
/* %endif */
	errno = oerrno;
}

/** Discard all buffered characters. On the next scan, YY_INPUT will be called.
 * @param b the buffer state to be flushed, usually @c YY_CURRENT_BUFFER.
 * @param yyscanner The scanner object.
 */
/* %if-c-only */
    void config__flush_buffer (YY_BUFFER_STATE  b , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if ( ! b )
		return;

	b->yy_n_chars = 0;

	/* We always need two end-of-buffer characters.  The first causes
	 * a transition to the end-of-buffer state.  The second causes
	 * a jam in that state.
	 */
	b->yy_ch_buf[0] = YY_END_OF_BUFFER_CHAR;
	b->yy_ch_buf[1] = YY_END_OF_BUFFER_CHAR;

	b->yy_buf_pos = &b->yy_ch_buf[0];

	b->yy_at_bol = 1;
	b->yy_buffer_status = YY_BUFFER_NEW;

	if ( b == YY_CURRENT_BUFFER )
		config__load_buffer_state(yyscanner );
}

/* %if-c-or-c++ */
/** Pushes the new state onto the stack. The new state becomes
 *  the current state. This function will allocate the stack
 *  if necessary.
 *  @param new_buffer The new state.
 *  @param yyscanner The scanner object.
 */
/* %if-c-only */
void config_push_buffer_state (YY_BUFFER_STATE new_buffer , yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if (new_buffer == NULL)
		return;

	config_ensure_buffer_stack(yyscanner);

	/* This block is copied from config__switch_to_buffer. */
	if ( YY_CURRENT_BUFFER )
		{
		/* Flush out information for old buffer. */
		*yyg->yy_c_buf_p = yyg->yy_hold_char;
		YY_CURRENT_BUFFER_LVALUE->yy_buf_pos = yyg->yy_c_buf_p;
		YY_CURRENT_BUFFER_LVALUE->yy_n_chars = yyg->yy_n_chars;
		}

	/* Only push if top exists. Otherwise, replace top. */
	if (YY_CURRENT_BUFFER)
		yyg->yy_buffer_stack_top++;
	YY_CURRENT_BUFFER_LVALUE = new_buffer;

	/* copied from config__switch_to_buffer. */
	config__load_buffer_state(yyscanner );
	yyg->yy_did_buffer_switch_on_eof = 1;
}
/* %endif */

/* %if-c-or-c++ */
/** Removes and deletes the top of the stack, if present.
 *  The next element becomes the new top.
 *  @param yyscanner The scanner object.
 */
/* %if-c-only */
void config_pop_buffer_state (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
	if (!YY_CURRENT_BUFFER)
		return;

	config__delete_buffer(YY_CURRENT_BUFFER ,yyscanner);
	YY_CURRENT_BUFFER_LVALUE = NULL;
	if (yyg->yy_buffer_stack_top > 0)
		--yyg->yy_buffer_stack_top;

	if (YY_CURRENT_BUFFER) {
		config__load_buffer_state(yyscanner );
		yyg->yy_did_buffer_switch_on_eof = 1;
	}
}
/* %endif */

/* %if-c-or-c++ */
/* Allocates the stack if it does not exist.
 *  Guarantees space for at least one push.
 */
/* %if-c-only */
static void config_ensure_buffer_stack (yyscan_t yyscanner)
/* %endif */
/* %if-c++-only */
/* %endif */
{
	int num_to_alloc;
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

	if (!yyg->yy_buffer_stack) {

		/* First allocation is just for 2 elements, since we don't know if this
		 * scanner will even need a stack. We use 2 instead of 1 to avoid an
		 * immediate realloc on the next call.
         */
		num_to_alloc = 1;
		yyg->yy_buffer_stack = (struct yy_buffer_state**)config_alloc
								(num_to_alloc * sizeof(struct yy_buffer_state*)
								, yyscanner);
		if ( ! yyg->yy_buffer_stack )
			YY_FATAL_ERROR( "out of dynamic memory in config_ensure_buffer_stack()" );
								  
		memset(yyg->yy_buffer_stack, 0, num_to_alloc * sizeof(struct yy_buffer_state*));
				
		yyg->yy_buffer_stack_max = num_to_alloc;
		yyg->yy_buffer_stack_top = 0;
		return;
	}

	if (yyg->yy_buffer_stack_top >= (yyg->yy_buffer_stack_max) - 1){

		/* Increase the buffer to prepare for a possible push. */
		int grow_size = 8 /* arbitrary grow size */;

		num_to_alloc = yyg->yy_buffer_stack_max + grow_size;
		yyg->yy_buffer_stack = (struct yy_buffer_state**)config_realloc
								(yyg->yy_buffer_stack,
								num_to_alloc * sizeof(struct yy_buffer_state*)
								, yyscanner);
		if ( ! yyg->yy_buffer_stack )
			YY_FATAL_ERROR( "out of dynamic memory in config_ensure_buffer_stack()" );

		/* zero only the new slots.*/
		memset(yyg->yy_buffer_stack + yyg->yy_buffer_stack_max, 0, grow_size * sizeof(struct yy_buffer_state*));
		yyg->yy_buffer_stack_max = num_to_alloc;
	}
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan directly from a user-specified character buffer.
 * @param base the character buffer
 * @param size the size in bytes of the character buffer
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object. 
 */
YY_BUFFER_STATE config__scan_buffer  (char * base, yy_size_t  size , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
    
	if ( size < 2 ||
	     base[size-2] != YY_END_OF_BUFFER_CHAR ||
	     base[size-1] != YY_END_OF_BUFFER_CHAR )
		/* They forgot to leave room for the EOB's. */
		return 0;

	b = (YY_BUFFER_STATE) config_alloc(sizeof( struct yy_buffer_state ) ,yyscanner );
	if ( ! b )
		YY_FATAL_ERROR( "out of dynamic memory in config__scan_buffer()" );

	b->yy_buf_size = size - 2;	/* "- 2" to take care of EOB's */
	b->yy_buf_pos = b->yy_ch_buf = base;
	b->yy_is_our_buffer = 0;
	b->yy_input_file = 0;
	b->yy_n_chars = b->yy_buf_size;
	b->yy_is_interactive = 0;
	b->yy_at_bol = 1;
	b->yy_fill_buffer = 0;
	b->yy_buffer_status = YY_BUFFER_NEW;

	config__switch_to_buffer(b ,yyscanner );

	return b;
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan a string. The next call to config_lex() will
 * scan from a @e copy of @a str.
 * @param yystr a NUL-terminated string to scan
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 * @note If you want to scan bytes that may contain NUL values, then use
 *       config__scan_bytes() instead.
 */
YY_BUFFER_STATE config__scan_string (yyconst char * yystr , yyscan_t yyscanner)
{
    
	return config__scan_bytes(yystr,strlen(yystr) ,yyscanner);
}
/* %endif */

/* %if-c-only */
/** Setup the input buffer state to scan the given bytes. The next call to config_lex() will
 * scan from a @e copy of @a bytes.
 * @param bytes the byte buffer to scan
 * @param len the number of bytes in the buffer pointed to by @a bytes.
 * @param yyscanner The scanner object.
 * @return the newly allocated buffer state object.
 */
YY_BUFFER_STATE config__scan_bytes  (yyconst char * yybytes, int  _yybytes_len , yyscan_t yyscanner)
{
	YY_BUFFER_STATE b;
	char *buf;
	yy_size_t n;
	int i;
    
	/* Get memory for full buffer, including space for trailing EOB's. */
	n = _yybytes_len + 2;
	buf = (char *) config_alloc(n ,yyscanner );
	if ( ! buf )
		YY_FATAL_ERROR( "out of dynamic memory in config__scan_bytes()" );

	for ( i = 0; i < _yybytes_len; ++i )
		buf[i] = yybytes[i];

	buf[_yybytes_len] = buf[_yybytes_len+1] = YY_END_OF_BUFFER_CHAR;

	b = config__scan_buffer(buf,n ,yyscanner);
	if ( ! b )
		YY_FATAL_ERROR( "bad buffer in config__scan_bytes()" );

	/* It's okay to grow etc. this buffer, and we should throw it
	 * away when we're done.
	 */
	b->yy_is_our_buffer = 1;

	return b;
}
/* %endif */

#ifndef YY_EXIT_FAILURE
#define YY_EXIT_FAILURE 2
#endif

/* %if-c-only */
static void yy_fatal_error (yyconst char* msg , yyscan_t yyscanner)
{
    	(void) fprintf( stderr, "%s\n", msg );
	exit( YY_EXIT_FAILURE );
}
/* %endif */
/* %if-c++-only */
/* %endif */

/* Redefine yyless() so it works in section 3 code. */

#undef yyless
#define yyless(n) \
	do \
		{ \
		/* Undo effects of setting up yytext. */ \
        int yyless_macro_arg = (n); \
        YY_LESS_LINENO(yyless_macro_arg);\
		yytext[yyleng] = yyg->yy_hold_char; \
		yyg->yy_c_buf_p = yytext + yyless_macro_arg; \
		yyg->yy_hold_char = *yyg->yy_c_buf_p; \
		*yyg->yy_c_buf_p = '\0'; \
		yyleng = yyless_macro_arg; \
		} \
	while ( 0 )

/* Accessor  methods (get/set functions) to struct members. */

/* %if-c-only */
/* %if-reentrant */

/** Get the user-defined data for this scanner.
 * @param yyscanner The scanner object.
 */
YY_EXTRA_TYPE config_get_extra  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyextra;
}

/* %endif */

/** Get the current line number.
 * @param yyscanner The scanner object.
 */
int config_get_lineno  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    
        if (! YY_CURRENT_BUFFER)
            return 0;
    
    return yylineno;
}

/** Get the current column number.
 * @param yyscanner The scanner object.
 */
int config_get_column  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    
        if (! YY_CURRENT_BUFFER)
            return 0;
    
    return yycolumn;
}

/** Get the input stream.
 * @param yyscanner The scanner object.
 */
FILE *config_get_in  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyin;
}

/** Get the output stream.
 * @param yyscanner The scanner object.
 */
FILE *config_get_out  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyout;
}

/** Get the length of the current token.
 * @param yyscanner The scanner object.
 */
int config_get_leng  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yyleng;
}

/** Get the current token.
 * @param yyscanner The scanner object.
 */

char *config_get_text  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yytext;
}

/* %if-reentrant */

/** Set the user-defined data. This data is never touched by the scanner.
 * @param user_defined The data to be associated with this scanner.
 * @param yyscanner The scanner object.
 */
void config_set_extra (YY_EXTRA_TYPE  user_defined , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyextra = user_defined ;
}

/* %endif */

/** Set the current line number.
 * @param line_number
 * @param yyscanner The scanner object.
 */
void config_set_lineno (int  line_number , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* lineno is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           yy_fatal_error( "config_set_lineno called with no buffer" , yyscanner); 
    
    yylineno = line_number;
}

/** Set the current column.
 * @param line_number
 * @param yyscanner The scanner object.
 */
void config_set_column (int  column_no , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

        /* column is only valid if an input buffer exists. */
        if (! YY_CURRENT_BUFFER )
           yy_fatal_error( "config_set_column called with no buffer" , yyscanner); 
    
    yycolumn = column_no;
}

/** Set the input stream. This does not discard the current
 * input buffer.
 * @param in_str A readable stream.
 * @param yyscanner The scanner object.
 * @see config__switch_to_buffer
 */
void config_set_in (FILE *  in_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyin = in_str ;
}

void config_set_out (FILE *  out_str , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yyout = out_str ;
}

int config_get_debug  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yy_flex_debug;
}

void config_set_debug (int  bdebug , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yy_flex_debug = bdebug ;
}

/* %endif */

/* %if-reentrant */
/* Accessor methods for yylval and yylloc */

/* %if-bison-bridge */

YYSTYPE * config_get_lval  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    return yylval;
}

void config_set_lval (YYSTYPE *  yylval_param , yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    yylval = yylval_param;
}

/* %endif */

/* User-visible API */

/* config_lex_init is special because it creates the scanner itself, so it is
 * the ONLY reentrant function that doesn't take the scanner as the last argument.
 * That's why we explicitly handle the declaration, instead of using our macros.
 */

int config_lex_init(yyscan_t* ptr_yy_globals)

{
    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }

    *ptr_yy_globals = (yyscan_t) config_alloc ( sizeof( struct yyguts_t ), NULL );

    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }

    /* By setting to 0xAA, we expose bugs in yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));

    return yy_init_globals ( *ptr_yy_globals );
}

/* config_lex_init_extra has the same functionality as config_lex_init, but follows the
 * convention of taking the scanner as the last argument. Note however, that
 * this is a *pointer* to a scanner, as it will be allocated by this call (and
 * is the reason, too, why this function also must handle its own declaration).
 * The user defined value in the first argument will be available to config_alloc in
 * the yyextra field.
 */

int config_lex_init_extra(YY_EXTRA_TYPE yy_user_defined,yyscan_t* ptr_yy_globals )

{
    struct yyguts_t dummy_yyguts;

    config_set_extra (yy_user_defined, &dummy_yyguts);

    if (ptr_yy_globals == NULL){
        errno = EINVAL;
        return 1;
    }
	
    *ptr_yy_globals = (yyscan_t) config_alloc ( sizeof( struct yyguts_t ), &dummy_yyguts );
	
    if (*ptr_yy_globals == NULL){
        errno = ENOMEM;
        return 1;
    }
    
    /* By setting to 0xAA, we expose bugs in
    yy_init_globals. Leave at 0x00 for releases. */
    memset(*ptr_yy_globals,0x00,sizeof(struct yyguts_t));
    
    config_set_extra (yy_user_defined, *ptr_yy_globals);
    
    return yy_init_globals ( *ptr_yy_globals );
}

/* %endif if-c-only */

/* %if-c-only */
static int yy_init_globals (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;
    /* Initialization is the same as for the non-reentrant scanner.
     * This function is called from config_lex_destroy(), so don't allocate here.
     */

    yyg->yy_buffer_stack = 0;
    yyg->yy_buffer_stack_top = 0;
    yyg->yy_buffer_stack_max = 0;
    yyg->yy_c_buf_p = (char *) 0;
    yyg->yy_init = 0;
    yyg->yy_start = 0;

    yyg->yy_start_stack_ptr = 0;
    yyg->yy_start_stack_depth = 0;
    yyg->yy_start_stack =  NULL;

/* Defined in main.c */
#ifdef YY_STDINIT
    yyin = stdin;
    yyout = stdout;
#else
    yyin = (FILE *) 0;
    yyout = (FILE *) 0;
#endif

    /* For future reference: Set errno on error, since we are called by
     * config_lex_init()
     */
    return 0;
}
/* %endif */

/* %if-c-only SNIP! this currently causes conflicts with the c++ scanner */
/* config_lex_destroy is for both reentrant and non-reentrant scanners. */
int config_lex_destroy  (yyscan_t yyscanner)
{
    struct yyguts_t * yyg = (struct yyguts_t*)yyscanner;

    /* Pop the buffer stack, destroying each element. */
	while(YY_CURRENT_BUFFER){
		config__delete_buffer(YY_CURRENT_BUFFER ,yyscanner );
		YY_CURRENT_BUFFER_LVALUE = NULL;
		config_pop_buffer_state(yyscanner);
	}

	/* Destroy the stack itself. */
	config_free(yyg->yy_buffer_stack ,yyscanner);
	yyg->yy_buffer_stack = NULL;

    /* Destroy the start condition stack. */
        config_free(yyg->yy_start_stack ,yyscanner );
        yyg->yy_start_stack = NULL;

    /* Reset the globals. This is important in a non-reentrant scanner so the next time
     * config_lex() is called, initialization will occur. */
    yy_init_globals( yyscanner);

/* %if-reentrant */
    /* Destroy the main struct (reentrant only). */
    config_free ( yyscanner , yyscanner );
    yyscanner = NULL;
/* %endif */
    return 0;
}
/* %endif */

/*
 * Internal utility routines.
 */

#ifndef yytext_ptr
static void yy_flex_strncpy (char* s1, yyconst char * s2, int n , yyscan_t yyscanner)
{
	register int i;
	for ( i = 0; i < n; ++i )
		s1[i] = s2[i];
}
#endif

#ifdef YY_NEED_STRLEN
static int yy_flex_strlen (yyconst char * s , yyscan_t yyscanner)
{
	register int n;
	for ( n = 0; s[n]; ++n )
		;

	return n;
}
#endif

void *config_alloc (yy_size_t  size , yyscan_t yyscanner)
{
	return (void *) malloc( size );
}

void *config_realloc  (void * ptr, yy_size_t  size , yyscan_t yyscanner)
{
	/* The cast to (char *) in the following accommodates both
	 * implementations that use char* generic pointers, and those
	 * that use void* generic pointers.  It works with the latter
	 * because both ANSI C and C++ allow castless assignment from
	 * any pointer type to void*, and deal with argument conversions
	 * as though doing an assignment.
	 */
	return (void *) realloc( (char *) ptr, size );
}

void config_free (void * ptr , yyscan_t yyscanner)
{
	free( (char *) ptr );	/* see config_realloc() for (char *) cast */
}

/* %if-tables-serialization definitions */
/* %define-yytables   The name for this specific scanner's tables. */
#define YYTABLES_NAME "yytables"
/* %endif */

/* %ok-for-header */

#line 203 "libmemcached/csl/scanner.l"



void Context::init_scanner()
{
  config_lex_init(&scanner);
  config_set_extra(this,scanner);
}

void Context::destroy_scanner()
{
  config_lex_destroy(scanner);
}


